@@ -1,5 +1,6 @@
-__package__ = "pynibs"
-
+"""
+All functions to operate on TMS coils go here, for example to create ``.xdmf`` files to visualize coil positions.
+"""
 import os
 import copy
 import math
@@ -23,14 +24,14 @@ def get_coil_dipole_pos(coil_fn, matsimnibs):
     Parameters
     ----------
     coil_fn : str
-        Filename of coil .ccd file
+        Filename of coil .ccd file.
     matsimnibs : np.ndarray of float
-        Transformation matrix
+        Transformation matrix.
 
     Returns
     -------
     dipoles_pos : np.ndarray
-    (N, 3) Cartesian coordinates (x, y, z) of coil magnetic dipoles
+        (N, 3) Cartesian coordinates (x, y, z) of coil magnetic dipoles.
 
     """
 
@@ -81,7 +82,7 @@ def check_coil_position(points, hull):
 
 def calc_coil_transformation_matrix(LOC_mean, ORI_mean, LOC_var, ORI_var, V):
     """
-    Calculate the modified coil transformation matrix needed for simnibs based on location and orientation
+    Calculate the modified coil transformation matrix needed for SimNIBS based on location and orientation
     variations observed in the framework of uncertainty analysis
 
     Parameters
@@ -156,22 +157,22 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
 
     Returns
     -------
-    pdf_paras_location : list of list of np.ndarrays [n_conditions]
-        Pdf parameters (limits and shape) of the coil position for x', y', and z' for each:
+    pdf_paras_location : list of list of np.ndarray
+        [n_conditions] Pdf parameters (limits and shape) of the coil position for x', y', and z' for each:
 
             - beta_paras ... [p, q, a, b] (2 shape parameters and limits)
             - moments    ... [data_mean, data_std, beta_mean, beta_std]
             - p_value    ... p-value of the Kolmogorov Smirnov test
             - uni_paras  ... [a, b] (limits)
-    pdf_paras_orientation_euler : list of np.ndarray [n_conditions]
-        Pdf parameters (limits and shape) of the coil orientation Psi, Theta, and Phi for each:
+    pdf_paras_orientation_euler : list of np.ndarray
+        [n_conditions] Pdf parameters (limits and shape) of the coil orientation Psi, Theta, and Phi for each:
 
             - beta_paras ... [p, q, a, b] (2 shape parameters and limits)
             - moments    ... [data_mean, data_std, beta_mean, beta_std]
             - p_value    ... p-value of the Kolmogorov Smirnov test
             - uni_paras  ... [a, b] (limits)
-    OP_mean : List of [3 x 4] np.ndarray [n_conditions]
-        List of mean coil position and orientation for different conditions (global coordinate system)
+    OP_mean : List of [3 x 4] np.ndarray
+        [n_conditions] List of mean coil position and orientation for different conditions (global coordinate system)
 
         .. math::
             \\begin{bmatrix}
@@ -311,7 +312,7 @@ def calc_coil_position_pdf(fn_rescon=None, fn_simpos=None, fn_exp=None, orientat
                 ylabel = [r'$p(\Psi)$', r'$p(\Theta)$', r'$p(\Phi)$']
 
             else:
-                ValueError('Please set orientation parameter to either ''quaternions'' or ''euler''')
+                raise ValueError('Please set orientation parameter to either "quaternions" or "euler"')
 
         # fit beta pdfs to x', y' and z' locations and orientations
         for j, name in enumerate(pos_names):
@@ -371,8 +372,6 @@ def test_coil_position_gpc(parameters):
             if p in ["x", "y", "z", "psi", "theta", "phi"]:
                 coil_uncertainty = True
 
-    # dim = len(parameters_random)
-
     if coil_uncertainty:
 
         svd_v = parameters["coil_position_mean"][0:3, 0:3]
@@ -476,24 +475,24 @@ def get_invalid_coil_parameters(param_dict, coil_position_mean, svd_v, del_obj,
 
     Parameters
     ----------
-    param_dict: dict
-        Dictionary containing dictionary with 'limits' and 'pdfshape'
-        keys: 'x', 'y', 'z', 'psi', 'theta', 'phi'
-    coil_position_mean: np.ndarray [3 x 4]
-        Mean coil positions and orientations
-    svd_v: np.ndarray [3 x 3]
-        SVD matrix V
-    del_obj: Delaunay object
-        Skin surface (created with scipy.spatial.Delaunay)
-    fn_coil: str
-        Filename of coil .ccd file
-    fn_hdf5_coilpos: str
-        Filename of .hdf5 file to save coil_pos in (incl. path and .hdf5 extension)
+    param_dict : dict
+        Dictionary containing dictionary with ``'limits'`` and ``'pdfshape'``.
+        keys: ``'x'``, ``'y'``, ``'z'``, ``'psi'``, ``'theta'``, ``'phi'``.
+    coil_position_mean: np.ndarray
+        (3, 4) Mean coil positions and orientations.
+    svd_v : np.ndarray
+        (3, 3) SVD matrix V.
+    del_obj : :class:`scipy.spatial.Delaunay`
+        Skin surface.
+    fn_coil : str
+        Filename of coil .ccd file.
+    fn_hdf5_coilpos : str
+        Filename of .hdf5 file to save coil_pos in (incl. path and .hdf5 extension).
 
     Returns
     -------
     fail_params: list of int
-        Index and combination of failed parameter
+        Index and combination of failed parameter.
     """
 
     # for every condition:
@@ -597,22 +596,22 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
     Parameters
     ----------
     fn_exp : str
-        Path to results.csv
+        Path to results.csv.
     fn_hdf : str
-        Path where to write file. Gets overridden if already existing
-    conditions_selected : str or list of str, Default=None
-        List of conditions returned by the function, the others are omitted,
-        If None, all conditions are returned
-    sep: str, Default: "_"
-        Separator between condition label and angle (e.g. M1_0, or M1-0)
-    merge_sites : boolean
+        Path where to write file. Gets overridden if already existing.
+    conditions_selected : str or list of str, optional
+        List of conditions returned by the function, the others are omitted.
+        If None, all conditions are returned.
+    sep: str, default: "_"
+        Separator between condition label and angle (e.g. M1_0, or M1-0).
+    merge_sites : bool
         If true, only one coil center per site is generated.
-    fix_angles : boolean
-        rename 22.5 -> 0, 0 -> -45, 67.5 -> 90, 90 -> 135
+    fix_angles : bool
+        rename 22.5 -> 0, 0 -> -45, 67.5 -> 90, 90 -> 135.
     data_dict : dict ofnp.ndarray of float [n_stimsites] (optional), default: None
-        Dictionary containing data corresponding to the stimulation sites (keys)
-    conditions_ignored : str or list of str, Default=None
-        Conditions, which are not going to be included in the plot
+        Dictionary containing data corresponding to the stimulation sites (keys).
+    conditions_ignored : str or list of str, optional
+        Conditions, which are not going to be included in the plot.
 
     Returns
     -------
@@ -670,7 +669,7 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
         m2.append(exp_cond[i_cond]['coil_mean_matrix'][0][0:3, 2])
 
     # split conds to angles and sites: M1_90 -> M1, 90
-    angles = np.array([sp.split(sep)[-1] for sp in conds]).astype(np.float)
+    angles = np.array([sp.split(sep)[-1] for sp in conds]).astype(np.float64)
     sites = np.array([sp.split(sep)[0] for sp in conds])
     sites_unique = np.unique(sites)
 
@@ -719,10 +718,10 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
     if not fn_hdf.endswith('.hdf5'):
         fn_hdf += '.hdf5'
     f = h5py.File(fn_hdf, 'w')
-    f.create_dataset('centers', data=centers.astype(np.float))
-    f.create_dataset('m0', data=m0.astype(np.float))
-    f.create_dataset('m1', data=m1.astype(np.float))
-    f.create_dataset('m2', data=m2.astype(np.float))
+    f.create_dataset('centers', data=centers.astype(np.float64))
+    f.create_dataset('m0', data=m0.astype(np.float64))
+    f.create_dataset('m1', data=m1.astype(np.float64))
+    f.create_dataset('m2', data=m2.astype(np.float64))
     f.create_dataset('cond', data=np.string_(conds))  # this is a string array, not xdmf compatible
     f.create_dataset('cond_idx', data=cond_idx)
     f.create_dataset('angles', data=angles)
@@ -809,36 +808,35 @@ def create_stimsite_hdf5(fn_exp, fn_hdf, conditions_selected=None, sep="_", merg
 
 
 def create_stimsite_from_list(fn_hdf, poslist, datanames=None, data=None, overwrite=False):
-    """This takes a TMSLIST from simnibs and creates a .hdf5 + .xdmf tuple for all positions.
+    """
+    This takes a list of matsimnibs-style coil position and orientations and creates an .hdf5 + .xdmf tuple
+    for all positions.
 
-    Centers and coil orientations are written so disk.
+    Centers and coil orientations are written to disk, with optional data for each coil configuration.
 
     Parameters
     ----------
     fn_hdf: str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-
-    datanames: str or list of str
-        Dataset names for _data_. Default: None.
-
-    data: np.ndarray
-        Dataset array with (len(poslist.pos), len(datanames()). Default: None.
-
-    poslist: TMSLIST object (simnibs.simulation.simstruct.TMSLIST)
-        poslist.pos[*].matsimnibs have to be set.
-
-    overwrite : boolean
-        Overwrite existing files. Default:  False.
+    poslist: list of np.ndarray
+        (4,4) Positions.
+    datanames: str or list of str, optional
+        Dataset names for ``data``.
+    data: np.ndarray, optional
+        Dataset array with shape = ``(len(poslist.pos), len(datanames())``.
+    overwrite : bool, defaul: False
+        Overwrite existing files.
     """
 
     centers = []
     m0 = []
     m1 = []
     m2 = []
+    if data is not None:
+        assert isinstance(data, np.ndarray)
 
     for lst in poslist:
-        # assert pos.matsimnibs is not None
         centers.append(lst[0:3, 3])
         m0.append(lst[0:3, 0])
         m1.append(lst[0:3, 1])
@@ -853,27 +851,24 @@ def create_stimsite_from_list(fn_hdf, poslist, datanames=None, data=None, overwr
 
 
 def create_stimsite_from_tmslist(fn_hdf, poslist, datanames=None, data=None, overwrite=False):
-    """This takes a TMSLIST from simnibs and creates a .hdf5 + .xdmf tuple for all positions.
+    """
+    This takes a :py:class:simnibs.sim_struct.TMSLIST from simnibs and creates an .hdf5 + .xdmf tuple for all positions.
 
-    Centers and coil orientations are written so disk.
+    Centers and coil orientations are written to disk, with optional data for each coil configuration.
 
     Parameters
     ----------
     fn_hdf: str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-
-    datanames: str or list of str
-        Dataset names for _data_. Default: None.
-
-    data: np.ndarray
-        Dataset array with (len(poslist.pos), len(datanames()). Default: None.
-
-    poslist: TMSLIST object (simnibs.simulation.simstruct.TMSLIST)
+    poslist: simnibs.sim_struct.TMSLIST
         poslist.pos[*].matsimnibs have to be set.
-
-    overwrite : boolean
-        Overwrite existing files. Default:  False.
+    datanames: str or list of str, optional
+        Dataset names for ``data``.
+    data: np.ndarray, optional
+        Dataset array with shape = ``(len(poslist.pos), len(datanames())``.
+    overwrite : bool, default: False
+        Overwrite existing files
     """
 
     centers = []
@@ -881,6 +876,8 @@ def create_stimsite_from_tmslist(fn_hdf, poslist, datanames=None, data=None, ove
     m1 = []
     m2 = []
     assert poslist.pos
+    if data is not None:
+        assert isinstance(data, np.ndarray)
     for pos in poslist.pos:
         assert pos.matsimnibs is not None
         pos.matsimnibs = np.array(pos.matsimnibs)
@@ -908,12 +905,12 @@ def create_stimsite_from_exp_hdf5(fn_exp, fn_hdf, datanames=None, data=None, ove
     fn_hdf : str
         Filename for the resulting .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-    datanames : str or list of str
-        Dataset names for _data_. Default: None.
-    data : np.ndarray
-        Dataset array with (len(poslist.pos), len(datanames()). Default: None.
-    overwrite : boolean
-        Overwrite existing files. Default:  False.
+    datanames : str or list of str, optional
+        Dataset names for ``data``
+    data : np.ndarray, optional
+        Dataset array with shape = ``(len(poslist.pos), len(datanames())``.
+    overwrite : bool, default: False
+        Overwrite existing files.
     """
 
     df_stim = pd.read_hdf(fn_exp, "stim_data")
@@ -952,18 +949,21 @@ def create_stimsite_from_matsimnibs(fn_hdf, matsimnibs, datanames=None, data=Non
             | & | & | & | \\\\
             0 & 0 & 0 & 1 \\\\
             \\end{bmatrix}
-    datanames: str or list of str
-        Dataset names for _data_. Default: None.
+    datanames: str or list of str, optional
+        Dataset names for ``data``.
     data: np.ndarray, optional
         (len(poslist.pos), len(datanames).
-    overwrite : boolean, default: False
+    overwrite : bool, default: False
         Overwrite existing files.
     """
+    matsimnibs = np.atleast_3d(matsimnibs)
     n_pos = matsimnibs.shape[2]
     centers = np.zeros((n_pos, 3))
     m0 = np.zeros((n_pos, 3))
     m1 = np.zeros((n_pos, 3))
     m2 = np.zeros((n_pos, 3))
+    if data is not None:
+        assert isinstance(data, np.ndarray)
 
     for i in range(matsimnibs.shape[2]):
         centers[i, :] = matsimnibs[0:3, 3, i]
@@ -976,43 +976,44 @@ def create_stimsite_from_matsimnibs(fn_hdf, matsimnibs, datanames=None, data=Non
 
 def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None, overwrite=False):
     """
-    Creates a .hdf5 + .xdmf file for all coil positions.
-
-    Centers and coil orientations are written to disk.
+    Creates a ``.hdf5`` + ``.xdmf`` file tuple for all coil positions.
+    Coil centers and coil orientations are saved, and - optionally - data for each position if ``data`` and
+    ``datanames`` are provided.
 
     Parameters
     ----------
     fn_hdf : str
         Filename for the .hdf5 file. The .xdmf is saved with the same basename.
         Folder should already exist.
-    centers : np.ndarray of float [n_pos x 3]
-        Coil positions
-    m0 : np.ndarray of float [n_pos x 3]
-        Coil orientation x-axis (looking at the active (patient) side of the coil pointing to the right)
-    m1 : np.ndarray of float [n_pos x 3]
-        Coil orientation y-axis (looking at the active (patient) side of the coil pointing up away from the handle)
-    m2 : np.ndarray of float [n_pos x 3]
-        Coil orientation z-axis (looking at the active (patient) side of the coil pointing to the patient)
-    datanames : str or list of str [n_data]
-        Dataset names for _data_. Default: None.
-    data : np.ndarray [n_pos, n_data]
-        Dataset array with (len(poslist.pos), len(datanames()). Default: None.
-    overwrite : boolean
-        Overwrite existing files. Default: False.
+    centers : np.ndarray of float
+        (n_pos, 3) Coil positions.
+    m0 : np.ndarray of float
+        (n_pos, 3) Coil orientation x-axis (looking at the active (patient) side of the coil pointing to the right).
+    m1 : np.ndarray of float
+        (n_pos, 3) Coil orientation y-axis (looking at the active side of the coil pointing up away from the handle).
+    m2 : np.ndarray of float
+        (n_pos, 3) Coil orientation z-axis (looking at the active (patient) side of the coil pointing to the patient).
+    datanames : str or list of str, optional
+        (n_data) Dataset names for ``data``
+    data : np.ndarray, optional
+        (n_pos, n_data) Dataset array with (len(poslist.pos), len(datanames()).
+    overwrite : bool, default: False
+        Overwrite existing files.
     """
     n_pos = centers.shape[0]
+    if type(datanames) == str:
+        datanames = [datanames]
 
     if data is not None:
         if datanames is None:
             raise ValueError("Provide datanames= with data= argument.")
-
-        if data.ndim == 1:
-            data = data[:, np.newaxis]
-
+        if type(datanames) == str:
+            datanames = [datanames]
+        if len(data.shape) <= 1:
+            data = np.atleast_1d(data)[:, np.newaxis]
         assert data.shape == (n_pos, len(datanames))
-
-    if type(datanames) == str:
-        datanames = [datanames]
+    if datanames is not None and data is None:
+        raise ValueError("Provide data= with datanames= argument.")
 
     m0_reshaped = np.hstack((m0, np.zeros((n_pos, 1)))).T[:, np.newaxis, :]
     m1_reshaped = np.hstack((m1, np.zeros((n_pos, 1)))).T[:, np.newaxis, :]
@@ -1027,72 +1028,70 @@ def write_coil_pos_hdf5(fn_hdf, centers, m0, m1, m2, datanames=None, data=None,
     if os.path.exists(fn_hdf) and not overwrite:
         raise OSError(fn_hdf + " already exists. Set overwrite flag for create_stimsite_from_poslist.")
 
-    f = h5py.File(fn_hdf, 'w')
-    f.create_dataset('centers', data=centers.astype(np.float))
-    f.create_dataset('m0', data=m0.astype(np.float))
-    f.create_dataset('m1', data=m1.astype(np.float))
-    f.create_dataset('m2', data=m2.astype(np.float))
-    f.create_dataset("matsimnibs", data=matsimnibs)
+    with h5py.File(fn_hdf, 'w') as f:
+        f.create_dataset('centers', data=centers.astype(np.float64))
+        f.create_dataset('m0', data=m0.astype(np.float64))
+        f.create_dataset('m1', data=m1.astype(np.float64))
+        f.create_dataset('m2', data=m2.astype(np.float64))
+        f.create_dataset("matsimnibs", data=matsimnibs)
 
-    if data is not None:
-        for i, col in enumerate(data.T):
-            f.create_dataset('/data/' + datanames[i], data=col)
-    f.close()
+        if data is not None:
+            for i, col in enumerate(data.T):
+                f.create_dataset('/data/' + datanames[i], data=col)
 
     # write .xdmf file
-    f = open(fn_hdf[:-4] + 'xdmf', 'w')
-    fn_hdf = os.path.basename(fn_hdf)  # relative links
-
-    # header
-    f.write('<?xml version="1.0"?>\n')
-    f.write('<!DOCTYPE Xdmf SYSTEM "Xdmf.dtd" []>\n')
-    f.write('<Xdmf Version="2.0" xmlns:xi="http://www.w3.org/2001/XInclude">\n')
-    f.write('<Domain>\n')
-    f.write('<Grid\nCollectionType="Spatial"\nGridType="Collection"\nName="Collection">\n')
-
-    # one grid for coil dipole nodes...store data hdf5.
-    #######################################################
-    f.write('<Grid Name="stimsites" GridType="Uniform">\n')
-    f.write('<Topology NumberOfElements="' + str(centers.shape[0]) +
-            '" TopologyType="Polyvertex" Name="Tri">\n')
-    f.write('<DataItem Format="XML" Dimensions="' + str(centers.shape[0]) + ' 1">\n')
-    np.savetxt(f, list(range(centers.shape[0])), fmt='%d', delimiter=' ')  # 1 2 3 4 ... N_Points
-    f.write('</DataItem>\n')
-    f.write('</Topology>\n')
-
-    # nodes
-    f.write('<Geometry GeometryType="XYZ">\n')
-    f.write('<DataItem Format="HDF" Dimensions="' + str(centers.shape[0]) + ' 3">\n')
-    f.write(fn_hdf + ':' + '/centers\n')
-    f.write('</DataItem>\n')
-    f.write('</Geometry>\n')
-
-    # data
-    # dipole magnitude
-    # the 4 vectors
-    for i in range(3):
-        f.write('<Attribute Name="dir_' + str(i) + '" AttributeType="Vector" Center="Cell">\n')
-        f.write('<DataItem Format="HDF" Dimensions="' + str(centers.shape[0]) + ' 3">\n')
-        f.write(fn_hdf + ':' + '/m' + str(i) + '\n')
+    with open(fn_hdf[:-4] + 'xdmf', 'w') as f:
+        fn_hdf = os.path.basename(fn_hdf)  # relative links
+    
+        # header
+        f.write('<?xml version="1.0"?>\n')
+        f.write('<!DOCTYPE Xdmf SYSTEM "Xdmf.dtd" []>\n')
+        f.write('<Xdmf Version="2.0" xmlns:xi="http://www.w3.org/2001/XInclude">\n')
+        f.write('<Domain>\n')
+        f.write('<Grid\nCollectionType="Spatial"\nGridType="Collection"\nName="Collection">\n')
+    
+        # one grid for coil dipole nodes...store data hdf5.
+        #######################################################
+        f.write('<Grid Name="stimsites" GridType="Uniform">\n')
+        f.write('<Topology NumberOfElements="' + str(centers.shape[0]) +
+                '" TopologyType="Polyvertex" Name="Tri">\n')
+        f.write('<DataItem Format="XML" Dimensions="' + str(centers.shape[0]) + ' 1">\n')
+        np.savetxt(f, list(range(centers.shape[0])), fmt='%d', delimiter=' ')  # 1 2 3 4 ... N_Points
         f.write('</DataItem>\n')
-        f.write('</Attribute>\n\n')
-
-    if data is not None:
-        for i, col in enumerate(data.T):
-            f.write(f'<Attribute Name="{datanames[i]}" AttributeType="Scalar" Center="Cell">\n')
-            f.write('<DataItem Format="HDF" Dimensions="' + str(centers.shape[0]) + ' 1">\n')
-            f.write(fn_hdf + f':/data/{datanames[i]}\n')
+        f.write('</Topology>\n')
+    
+        # nodes
+        f.write('<Geometry GeometryType="XYZ">\n')
+        f.write('<DataItem Format="HDF" Dimensions="' + str(centers.shape[0]) + ' 3">\n')
+        f.write(fn_hdf + ':' + '/centers\n')
+        f.write('</DataItem>\n')
+        f.write('</Geometry>\n')
+    
+        # data
+        # dipole magnitude
+        # the 4 vectors
+        for i in range(3):
+            f.write('<Attribute Name="dir_' + str(i) + '" AttributeType="Vector" Center="Cell">\n')
+            f.write('<DataItem Format="HDF" Dimensions="' + str(centers.shape[0]) + ' 3">\n')
+            f.write(fn_hdf + ':' + '/m' + str(i) + '\n')
             f.write('</DataItem>\n')
             f.write('</Attribute>\n\n')
-
-    f.write('</Grid>\n')
-    # end coil dipole data
-
-    # footer
-    f.write('</Grid>\n')
-    f.write('</Domain>\n')
-    f.write('</Xdmf>\n')
-    f.close()
+    
+        if data is not None:
+            for i, col in enumerate(data.T):
+                f.write(f'<Attribute Name="{datanames[i]}" AttributeType="Scalar" Center="Cell">\n')
+                f.write('<DataItem Format="HDF" Dimensions="' + str(centers.shape[0]) + ' 1">\n')
+                f.write(fn_hdf + f':/data/{datanames[i]}\n')
+                f.write('</DataItem>\n')
+                f.write('</Attribute>\n\n')
+    
+        f.write('</Grid>\n')
+        # end coil dipole data
+    
+        # footer
+        f.write('</Grid>\n')
+        f.write('</Domain>\n')
+        f.write('</Xdmf>\n')
 
 
 def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root_path="/0/0/", verbose=False,
@@ -1108,9 +1107,9 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
         Name of .hdf5 file containing the matsimnibs matrices of all coil positions
     fn_out_hdf5 : str
         Name of output .hdf5 file (will be saved in the same format as fn_coil_pos_opt)
-    verbose : bool, optional, default: False
+    verbose : bool, default: False
         Print output messages
-    print_output : bool or str, optional, default: False
+    print_output : bool or str, default: False
         Print output image as .png file showing optimal path
 
     Returns
@@ -1282,7 +1281,6 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
         else:
             f[root_path + "coil_seq"][:] = np.vstack((idx_global_sorted, goal_fun_val_sorted)).transpose()
 
-
     if print_output:
         fig = plt.figure()
         ax = fig.add_subplot(projection='3d')
@@ -1296,314 +1294,3 @@ def sort_opt_coil_positions(fn_coil_pos_opt, fn_coil_pos, fn_out_hdf5=None, root
         ax.set_zlabel("z in mm")
         ax.view_init(elev=40., azim=230)
         plt.savefig(print_output, dpi=600, transparent=True)
-
-# def check_valid_coil_position_uncertainty(self, problem, coil_position_mean, svd_v):
-#     """ Tests extreme cases of possible coil positions if dipoles are located inside the head.
-#     If an invalid position is found, the parameter space is reduced by shrinking the limits.
-#
-#     Parameters:
-#     ----------
-#     problem : Problem class instance
-#         GPC problem under investigation
-#     coil_position_mean : np.ndarray of float [3 x 4]
-#         Mean coil position matrix containing the 3 orientation vectors and the location [ori_x, ori_y, ori_z, loc]
-#     svd_v: np.ndarray of float [3 x 3]
-#         List containing the SVD matrices V for each condition (principal axes of orientation)
-#
-#     Returns:
-#     --------
-#     param_dict: list of dict [n_conditions]
-#         List of dictionaries containing dictionary with corrected 'limits' and 'pdfshape'
-#         keys: 'x', 'y', 'z', 'psi', 'theta', 'phi'
-#    """
-#
-#     # Loading geometry to create Delaunay object of outer skin surface
-#     with h5py.File(self.subject.mesh[self.mesh_idx]['fn_mesh_hdf5'], 'r') as f:
-#         points = np.array(f['mesh/nodes/node_coord'])
-#         node_number_list = np.array(f['mesh/elm/node_number_list'])
-#         elm_type = np.array(f['mesh/elm/elm_type'])
-#         regions = np.array(f['mesh/elm/tag1'])
-#         triangles_regions = regions[elm_type == 2, ] - 1000
-#         triangles = node_number_list[elm_type == 2, 0:3]
-#
-#     triangles = triangles[triangles_regions == 5]
-#     surface_points = pynibs.pynibs.unique_rows(np.reshape(points[triangles], (3 * triangles.shape[0], 3)))
-#     limits_scaling_factor = .1
-#
-#     # Generate Delaunay triangulation object
-#     dobj = Delaunay(surface_points)
-#
-#     del points, node_number_list, elm_type, regions, triangles_regions, triangles, surface_points
-#
-#     # initialize parameter dictionary
-#     param_dict = dict()
-#     param_dict_idx = dict()
-#     coil_param = ['x', 'y', 'z', 'psi', 'theta', 'phi']
-#
-#     for i, c in enumerate(coil_param):
-#         param_dict[c] = {'pdf_limits': [0, 0], 'pdf_shape': [0, 0]}
-#         param_dict_idx[i] = c
-#
-#     # we need to map some indices
-#     dic_par_name = dict.fromkeys([0, 1, 2], "pdf_paras_location")
-#     dic_par_name.update(dict.fromkeys([3, 4, 5], "pdf_paras_orientation_euler"))
-#     dic_limit_id = {0: 0, 2: 1}  # translates from get_invalid_coil_parameters limits id to pdf_paras_* id
-#     dic_param_id = dict.fromkeys([0, 3], 0)
-#     dic_param_id.update(dict.fromkeys([1, 4], 1))
-#     dic_param_id.update(dict.fromkeys([2, 5], 2))
-#
-#     # Determine bad parameters and correct them
-#     print("Checking validity of coil position uncertainty for condition " + self.cond)
-#     random.seed(1)
-#
-#     # gather random variables in parameter dict (constants = 0 = no variation)
-#     for i_rv, rv in enumerate(problem.random_vars):
-#         if rv in coil_param:
-#             param_dict[rv] = {'pdf_limits': problem.parameters[rv].pdf_limits,
-#                               'pdf_shape': problem.parameters[rv].pdf_shape}
-#
-#     bad_params = self.get_invalid_coil_parameters(param_dict,
-#                                                   coil_position_mean=coil_position_mean,
-#                                                   svd_v=svd_v,
-#                                                   del_obj=dobj,
-#                                                   fn_coil=self.fn_coil)
-#
-#     # change param limits until no dipoles left inside head
-#     while bad_params:
-#         print(" > Invalid parameter found!")
-#         # several parameters may lead to a bad position
-#         # take a random parameter from these
-#         param_id = random.sample(range(len(bad_params)), 1)[0]
-#
-#         # change last bad_params[-1]' limit
-#         param_to_change_idx = bad_params[param_id][0]
-#         limit_to_change_idx = dic_limit_id[bad_params[param_id][1]]  # change min or max limit from 3 to 2 limit
-#
-#         # grab old limits of parameter to change
-#         limits_old = param_dict[param_dict_idx[param_to_change_idx]]['pdf_limits']
-#
-#         # rescale limits (but change only important boundary, min or max)
-#         factor = (limits_old[1] - limits_old[0]) * limits_scaling_factor
-#         limits_temp = [limits_old[0] + factor, limits_old[1] - factor]
-#         limits_new = copy.deepcopy(limits_old)
-#         limits_new[limit_to_change_idx] = limits_temp[limit_to_change_idx]
-#         param_dict[param_dict_idx[param_to_change_idx]]['pdf_limits'] = limits_new
-#
-#         print(' > Changing pdf_limits of {} from {} -> {}'.format(param_dict_idx[param_to_change_idx],
-#                                                              limits_old,
-#                                                              limits_new))
-#
-#         # repeat dipole check
-#         bad_params = self.get_invalid_coil_parameters(param_dict,
-#                                                       coil_position_mean=self.coil_position_mean,
-#                                                       svd_v=svd_v,
-#                                                       del_obj=dobj,
-#                                                       fn_coil=self.fn_coil)
-#
-#     return param_dict
-
-# def calc_coil_position_pdf(fn_rescon, fn_simpos, folder_pdfplots):
-#     """ Determines the probability density functions of the transformed coil position (x', y', z') and Euler angles of
-#     the coil orientations (Psi, Theta, Phi)
-#
-#     calc_coil_position_pdf(fn_rescon, fn_simpos)
-#
-#     Parameters
-#     ----------
-#     fn_rescon: str
-#         Filename of the results file from TMS experiments (results_conditions.csv)
-#     fn_simpos: str
-#         Filename of the positions and orientation from TMS experiments (simPos.csv)
-#     folder_pdfplots: str
-#         Folder, where the plots of the fitted pdfs are saved
-#
-#     Returns
-#     -------
-#     pdf_paras_location: list of np.array [n_conditions]
-#         Pdf parameters (limits and shape) of the coil position
-#     pdf_paras_orientation_euler: list of np.array [n_conditions]
-#         Pdf parameters (limits and shape) of the coil orientation
-#     positions_mean: List of [3x3] np.array [n_conditions]
-#         List of mean coil position and orientation for different conditions (global coordinate system)
-#     locations_zeromean: list of np.array [n_conditions]
-#         List of shifted coil positions (zero-mean) for different conditions
-#     V: list of [3x3] np.arrays [n_conditions]
-#         Transformation matrix of coil positions from global coordinate system to transformed coordinate system
-#     locations_transform: np.array [N_zaps_total x 3]
-#         Transformed coil positions [x', y', z'] of all stimulations (zero-mean, rotated by SVD)
-#     euler_angles: np.array [N_zaps_total x 3]
-#         Euler angles [Psi, Theta, Phi] of all stimulations
-#     """
-#     import pygpc
-#
-#     if folder_pdfplots is not None:
-#         make_pdf_plot = True
-#     else:
-#         make_pdf_plot = False
-#         folder_pdfplots = ''
-#
-#     # load recorded positions and orientations
-#     positions_all, conditions, position_list, _, _ = pynibs.read_exp_stimulations(fn_rescon, fn_simpos)
-#
-#     # sort POSITIONS according to CONDITIONS, idx_con is alphabetically sorted, first index of condition[*]
-#     appeareance
-#     conditions_unique, idx_con, n_con_each = np.unique(conditions,
-#                                                        return_index=True,
-#                                                        return_inverse=False,
-#                                                        return_counts=True)
-#     n_condition = len(conditions_unique)
-#     n_zaps = len(positions_all)
-#
-#     idx_CON_sort = np.argsort(idx_con)
-#     # idx_con = idx_con[idx_CON_sort]
-#     conditions_unique = conditions_unique[idx_CON_sort]  # conditions_unique is ordered like exp. data
-#     n_con_each = n_con_each[idx_CON_sort]
-#
-#     positions = [[] for i in range(n_condition)]
-#     ori_x = [0] * n_condition
-#     ori_y = [0] * n_condition
-#     ori_z = [0] * n_condition
-#     loc = [0] * n_condition
-#     i = 0
-#     k = 0
-#     for j in range(n_condition):
-#         ori_x[j] = np.zeros((n_con_each[j], 3))
-#         ori_y[j] = np.zeros((n_con_each[j], 3))
-#         ori_z[j] = np.zeros((n_con_each[j], 3))
-#         loc[j] = np.zeros((n_con_each[j], 3))
-#
-#         while position_list[i][len(position_list[0])-3] == conditions_unique[j]:
-#             positions[j].append(np.array(positions_all[i])[0:3, 0:4])
-#             ori_x[j][k, ] = np.array(positions_all[i])[0:3, 0]
-#             ori_y[j][k, ] = np.array(positions_all[i])[0:3, 1]
-#             ori_z[j][k, ] = np.array(positions_all[i])[0:3, 2]
-#             loc[j][k, ] = np.array(positions_all[i])[0:3, 3]
-#             i = i + 1
-#             k = k + 1
-#             if i == n_zaps:
-#                 break
-#         k = 0
-#     del i, j
-#
-#     # determine "mean" location and orientation
-#     positions_mean = [0]*n_condition
-#     pos = np.zeros((3, 4))
-#
-#     for i in range(n_condition):
-#         for j in range(n_con_each[i]):
-#             pos = pos + positions[i][j]
-#         pos = pos / n_con_each[i]
-#         positions_mean[i] = pos  # POSITIONS_mean[0] is mean coil center in condition '0'
-#         pos = np.zeros((3, 4))
-#
-#         # test orthogonality of mean orientation axis
-#         xy_orth = np.inner(positions_mean[i][:, 0], positions_mean[i][:, 1])
-#         xz_orth = np.inner(positions_mean[i][:, 0], positions_mean[i][:, 2])
-#         yz_orth = np.inner(positions_mean[i][:, 1], positions_mean[i][:, 2])
-#
-#     del i, j
-#
-#     # determine location inaccuracy
-#
-#     # concatenate all locations into one array
-#     locations = [np.zeros((3,1))]*n_condition
-#     for i in range(n_condition):
-#         for j in range(n_con_each[i]):
-#             locations[i] = np.hstack((locations[i], positions[i][j][0:3, 3][:, np.newaxis]))
-#         locations[i] = np.delete(locations[i], 0, axis=1)
-#
-#     # shift points for SVD
-#     locations_zeromean = [0]*n_condition
-#     locations_transform = [0]*n_condition
-#     U = [0]*n_condition
-#     s = [0]*n_condition
-#     V = [0]*n_condition
-#
-#     for i in range(n_condition):
-#         locations_zeromean[i] = locations[i] - np.tile(positions_mean[i][:, 3][:, np.newaxis],
-#                                                        (1, n_con_each[i]))
-#         # LOCATIONS_zeromean[0] are 'normalized' positions
-#
-#         # perform SVD to determine principal axis
-#         U[i], s[i], V[i] = np.linalg.svd(locations_zeromean[i].T, full_matrices=True)
-#
-#         # project locations on principal axis
-#         locations_transform[i] = np.dot(locations_zeromean[i].T, V[i].T)
-#         # LOCATIONS_transform[0] are svd'ed, normalized positions of condition 0
-#
-#     # fit beta pdfs to x', y' and z' locations and
-#     pdf_paras_location = [[[] for i in range(3)] for j in range(n_condition)]
-#     np.random.seed(5)
-#     loc_labels = ['xp', 'yp', 'zp']
-#     for i in range(n_condition):
-#         for j in range(3):
-#             pdf_paras_location[i][j] = pygpc.fit_betapdf(locations_transform[i][:, j],
-#                                                          BETATOL=0.05,
-#                                                          PUNI=0.9,
-#                                                          PLOT=make_pdf_plot,
-#                                                          VISI=0,
-#                                                          filename=os.path.join(folder_pdfplots, 'POS_' +
-#                                                                                conditions_unique[i] + '_' +
-#                                                                                loc_labels[j] +
-#                                                                                '_betafit'))
-#         # pdf_paras_location[0][0] are beta parameter for condition 0, X axis
-#
-#     # project coil orientations to mean orientation
-#
-#     ori_x_transform = [0]*n_condition
-#     ori_y_transform = [0]*n_condition
-#     ori_z_transform = [0]*n_condition
-#
-#     for i in range(n_condition):
-#         # for j in range(N_CON_each[i]):
-#         ori_x_transform[i] = np.dot(ori_x[i], positions_mean[i][0:3, 0:3])
-#         ori_y_transform[i] = np.dot(ori_y[i], positions_mean[i][0:3, 0:3])
-#         ori_z_transform[i] = np.dot(ori_z[i], positions_mean[i][0:3, 0:3])
-#
-#     # # plot original orientations (endpoints) of first axis of first condition
-#     # fig = plt.figure()
-#     # ax = fig.add_subplot(111, projection='3d')
-#     # ax.scatter(ORI_X[0][:,0], ORI_X[0][:,1], ORI_X[0][:,2] , zdir='z', s=20, c=[1,1,0], depthshade=True)
-#     # fig.savefig(folder + 'ORI_X_0.png')
-#     #
-#     # fig = plt.figure()
-#     # ax = fig.add_subplot(111, projection='3d')
-#     # ax.scatter(ORI_X_transform[0][:,0], ORI_X_transform[0][:,1], ORI_X_transform[0][:,2] , zdir='z',
-#     s=20, c=[1,1,0],
-#     #  depthshade=True)
-#     # fig.savefig(folder + 'ORI_X_transform_0.png')
-#
-#     # estimate angle variation between axes
-#     pdf_paras_orientation = [[[] for i in range(3)] for j in range(n_condition)]
-#     pdf_paras_orientation_euler = [[[] for i in range(3)] for j in range(n_condition)]
-#     deviation_angle_all = [0]*n_condition
-#     angle_names = ['Psi', 'Theta', 'Phi']
-#     euler_angles = [0]*n_condition
-#
-#     r_all = []  # store all R here
-#     for i in range(n_condition):
-#         deviation_angle_all[i] = np.zeros((n_con_each[i], 3))
-#
-#         # Determine Euler angles
-#         euler_angles[i] = np.zeros((n_con_each[i], 3))
-#
-#         for j in range(n_con_each[i]):
-#             R = np.hstack((ori_x_transform[i][j, :][:, np.newaxis],
-#                            ori_y_transform[i][j, :][:, np.newaxis],
-#                            ori_z_transform[i][j, :][:, np.newaxis]))
-#             euler_angles[i][j, :] = pynibs.rotation_matrix_to_euler_angles(R) * 180.0 / np.pi
-#             r_all.append(R)
-#         for j, name in enumerate(angle_names):
-#             np.random.seed(5)
-#             pdf_paras_orientation_euler[i][j] = pygpc.fit_betapdf(euler_angles[i][:, j],
-#                                                                   BETATOL=0.05,
-#                                                                   PUNI=0.9,
-#                                                                   PLOT=make_pdf_plot,
-#                                                                   VISI=0,
-#                                                                   filename=os.path.join(folder_pdfplots, 'ORI_' +
-#                                                                                         conditions_unique[i] + '_' +
-#                                                                                         angle_names[j] +
-#                                                                                         '_betafit'))
-#             # pdf_paras_orientation_euler[0][0] are beta parameter for condition 0, alpha (pitch)
-#     return pdf_paras_location, pdf_paras_orientation_euler, positions_mean, \
-#            locations_zeromean, V, np.concatenate(locations_transform), np.concatenate(euler_angles)
