@@ -655,7 +655,7 @@ def cf_variance_workhorse(elm_idx_list, mep, mep_params, e, old_style=True):
         * ``len(e) = n_cond``
         * ``len(e[0]) = n_comp`` (e.g: ``e_mag = e[0])``)
 
-    old_style: boolean (default: True)
+    old_style: bool, default: True
         True:  Weight ``var(x_0_prime(r))`` with ``mean(e(r) * mean(Stimulator Intensity)``, taken from ``mep``
         False: Weight ``var(x_0_prime(r))`` with ``mean(E(r))``, taken from `e`
 
