@@ -82,7 +82,16 @@ def sigmoid4(x, x0, r, amp, y0):
     y : np.ndarray of float
         (N_x) Function value at argument x.
     """
-    y = y0 + (amp - y0) / (1 + np.exp(-r * (x - x0)))
+    exp = -r * (x - x0)
+
+    # avoid numpy RuntimeWarning: overflow encountered in exp
+    if np.all(exp < 500):
+        y = y0 + (amp - y0) / (1 + np.exp(exp))
+    else:
+        # return a flat line; high value to maximally impair fits
+        y = np.ones(x.shape)
+        y *= np.iinfo(np.int32).max
+
     return y
 
 
@@ -386,7 +395,7 @@ def get_mep_virtual_subject_TVS(x, p1=-5.0818, p2=-2.4677, p3=3.6466, p4=0.42639
     return mep
 
 
-def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0=1e-2, seed=None):
+def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0=1e-2, seed=None, rng=None):
     """
     Creates random mep data using the 2 variability source model from Goetz et al. 2014 [1]_ together with a standard 3
     parametric sigmoid function.
@@ -415,14 +424,24 @@ def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0
         y-offset.
     seed : int, default: None
         Seed to use.
+    rng : numpy._generator.Generator, default: None
+        An already initialized random number generator that will be used instead of intializing a new one with seed.
 
     Returns
     -------
     mep : np.ndarray of float
         (n_x) Motor evoked potential values
     """
-    if seed:
-        np.random.seed(seed)
+
+    if type(rng) is np.random._generator.Generator:
+        default_rng = rng
+    else:
+        if type(rng) is not type(None):
+            print("[get_mep_virtual_subject_DVS] Invalid argument: get_mep_virtual_subject_DVS. Using numpy default_rng"
+                  f" with seed={seed}")
+
+        default_rng = np.random.default_rng(seed=seed)
+
 
     if not isinstance(x, np.ndarray):
         x = np.array([x])
@@ -430,12 +449,12 @@ def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0
     n_x = len(x)
 
     if sigma_x > 0:
-        e_x_add = np.random.normal(loc=0, scale=sigma_x, size=n_x)
+        e_x_add = default_rng.normal(loc=0, scale=sigma_x, size=n_x)
     else:
         e_x_add = np.zeros(n_x)
 
     if sigma_y > 0:
-        e_y_add = np.random.normal(loc=0, scale=sigma_y, size=n_x)
+        e_y_add = default_rng.normal(loc=0, scale=sigma_y, size=n_x)
     else:
         e_y_add = np.zeros(n_x)
 
@@ -447,8 +466,6 @@ def get_mep_virtual_subject_DVS(x, x0=0.5, r=10, amp=1, sigma_x=0, sigma_y=0, y0
 
 
 class Mep:
-    # TODO: implement list of already fitted function types and their indices in self.fit[] for plot function. \
-    #       condition name as attribute
     """
     Mep object.
 
@@ -459,23 +476,24 @@ class Mep:
     fun_sig : function
         Best fitting equivalent sigmoidal function (added by ``self.add_sigmoidal_bestfit()``).
     popt : np.ndarray of float
-        (N_para), Fitted optimal function parameters.
+        (N_para) Fitted optimal function parameters.
     popt_sig : np.ndarray of float
-        (3), Best fitting parameters ``x0``, ``r``, and ``amp`` of equivalent sigmoidal function.
+        (3) Best fitting parameters ``x0``, ``r``, and ``amp`` of equivalent sigmoidal function.
     copt :  np.ndarray of float
-        (N_para, N_para), covariance matrix of fitted parameters.
+        (N_para, N_para) covariance matrix of fitted parameters.
     pstd : np.ndarray of float
-        (N_para), Standard deviation of fitted parameters.
+        (N_para) Standard deviation of fitted parameters.
     fit : object instance
         Gmodel object instance of parameter fit.
     x_limits : np.ndarray of float
-        (2), Minimal and maximal value of intensity data.
+        (2) Minimal and maximal value of intensity data.
     y_limits : np.ndarray of float
-        (2), Minimal and maximal value of mep data.
+        (2) Minimal and maximal value of mep data.
     mt : float
         Motor threshold (MEP > 50 uV), evaluated from fitted curve, added after fitting.
     """
-
+    # TODO: implement list of already fitted function types and their indices in self.fit[] for plot function. \
+    #       condition name as attribute
     def __init__(self, intensities, mep, intensity_min_threshold=None, mep_min_threshold=None):
         """
         Initialize ``Mep`` object.
@@ -483,9 +501,9 @@ class Mep:
         Parameters
         ----------
         intensities : np.ndarray of float
-            (N_mep), Intensities of TMS stimulator corresponding to measured MEP amplitudes.
+            (N_mep) Intensities of TMS stimulator corresponding to measured MEP amplitudes.
         mep : np.ndarray of float
-            (N_mep), MEP amplitudes in [V] measured during TMS.
+            (N_mep) MEP amplitudes in [V] measured during TMS.
         intensity_min_threshold : float
             Minimum user defined intensity (values below it will be filtered out).
         mep_min_threshold : float
@@ -540,7 +558,7 @@ class Mep:
 
         Notes
         -----
-        Add Attributes:
+        Additional attributes:
 
         Mep.popt : np.ndarray of float
             (N_para) Fitted optimal function parameters.
@@ -736,9 +754,9 @@ class Mep:
         fun : function
             Function mep data has to be fitted with
         x : np.ndarray of float
-            (N_data), Independent variable the data is fitted on.
+            (N_data) Independent variable the data is fitted on.
         y : np.ndarray of float
-            (N_data), Dependent data the curve is fitted through.
+            (N_data) Dependent data the curve is fitted through.
         p0 : np.ndarray of float or list of float
             Initial guess of parameter values.
         constraints : dict, optional
@@ -859,9 +877,9 @@ class Mep:
             Plot title.
         sigma : float
             Factor of standard deviations the uncertainty of the fit is plotted with.
-        plot_samples : boolean
+        plot_samples : bool
             Plot sampling curves of the fit in the uncertainty interval.
-        show_plot : boolean
+        show_plot : bool
             Show or hide plot window (``TRUE`` / ``FALSE``).
         fname_plot : str
             Filename of plot showing fitted data (with .png extension).
@@ -1147,9 +1165,9 @@ def get_mep_elements(mep_fn, tms_pulse_time, drop_mep_idx=None, cfs_data_column=
         Channel names.
     time_format : str, default:"delta"
         Format of mep time stamps in time_mep_lst to return.
-
         * ``"delta"`` returns list of datetime.timedelta in seconds
         * ``"hms"`` returns datetime.datetime(year, month, day, hour, minute, second, microsecond)
+
     plot : bool, default: False
         Plot MEPs.
     start_mep : float, default: 18
@@ -1603,7 +1621,9 @@ def get_mep_sampling_rate(cfs_path):
     Returns sampling rate [Hz] for CED Signal EMG data in .cfs, .mat or .txt file.
 
     The sampling rate is saved in the cfs header like this:
+
     .. code-block:: sh
+
        ``Samplingrate"\t: 3999.999810,\n``
 
     Parameters
