@@ -253,7 +253,8 @@ def create_merged_nnav_emg_file_brainsight(
 
 def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond=False,
                               remove_coil_skin_distance_outlier=True, coil_distance_corr=True,
-                              verbose=False, plot=False, start_mep=18, end_mep=35):
+                              average_emg_data_from_same_condition=False,verbose=False, plot=False,
+                              start_mep=18, end_mep=35):
     """
     Merge the TMS coil positions and the mep data into an experiment.hdf5 file.
 
@@ -292,29 +293,34 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
                             "nnav2simnibs",
                             f"mesh_{mesh_idx}")
     fn_data = None
-    if isinstance(subject.exp[exp_idx]["fn_data"], list):
-        try:
-            fn_data = subject.exp[exp_idx]["fn_data"][0][0]
-        except IndexError as e:
-            fn_data = subject.exp[exp_idx]["fn_data"][0]
-    elif isinstance(subject.exp[exp_idx]["fn_data"], str):
-        fn_data = subject.exp[exp_idx]["fn_data"]
+    if "fn_data" in subject.exp[exp_idx].keys():
+        if isinstance(subject.exp[exp_idx]["fn_data"], list):
+            if isinstance(subject.exp[exp_idx]["fn_data"][0], str):
+                fn_data = subject.exp[exp_idx]["fn_data"][0]
+            else:
+                fn_data = subject.exp[exp_idx]["fn_data"][0][0]
+        elif isinstance(subject.exp[exp_idx]["fn_data"], str):
+            fn_data = subject.exp[exp_idx]["fn_data"]
+        else:
+            print("[Error] Invalid exp[*]['fn_data']: "
+                  "either specify the path as a single string or as a nested list of strings [['path']]!"
+                  "Will attempt to read EMG data from exp[*]['fn_tms_nav']")
     else:
-        print("[Error] Invalid exp[*]['fn_data']: "
-              "either specify the path as a single string or as a nested list of strings [['path']]!"
-              "Will attempt to read EMG data from exp[*]['fn_tms_nav']")
+        print("[Warn] No EMG data specified in exp[*]['fn_data']: Will read EMG data from exp[*]['fn_tms_nav']")
 
-    if isinstance(subject.exp[exp_idx]["fn_tms_nav"], list):
-        try:
+    if "fn_tms_nav" in subject.exp[exp_idx]:
+        if isinstance(subject.exp[exp_idx]["fn_tms_nav"], list):
+            if isinstance(subject.exp[exp_idx]["fn_tms_nav"][0], str):
+                fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0]
+            else:
+                fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0][0]
+        elif isinstance(subject.exp[exp_idx]["fn_tms_nav"], str):
             fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0][0]
-        except IndexError as e:
-            fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0]
-    elif isinstance(subject.exp[exp_idx]["fn_tms_nav"], str):
-        fn_tms_nav = subject.exp[exp_idx]["fn_tms_nav"][0][0]
+        else:
+            raise ValueError("Invalid exp[*]['fn_tms_nav']: "
+                             "Either specify the path as a single string or as a nested list of strings [['path']]!")
     else:
-        print("[Fatal error] Invalid exp[*]['fn_tms_nav']: "
-              "either specify the path as a single string or as a nested list of strings [['path']]!")
-        return 1
+        raise KeyError("No exp[*]['fn_tms_nav'] found in subject object. Cannot continue to merge data,")
 
     if fn_data is not None:
         fn_out, out_ext = os.path.splitext(fn_tms_nav)
@@ -358,50 +364,56 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
     for key in keys:
         d_bs[key] = []
 
-    # collect data
-    for i_loc, i_glo in enumerate(range(start_idx, stop_idx)):
-        line = lines[i_glo].split(sep='\t')
+    # collect data; for each line...
+    for idx_coil_sample, idx_line in enumerate(range(start_idx, stop_idx)):
+        line = lines[idx_line].split(sep='\t')
 
-        for i_key, key in enumerate(d_bs.keys()):
+        # ...populate the keys of the brainsight-dict
+        for idx_key, key in enumerate(d_bs.keys()):
+            # standard (string) entries can be appended straightforwardly
             if key in ["Sample Name", "Session Name", "Creation Cause", "Crosshairs Driver", "Date", "Time",
                        "EMG Channels", "Assoc. Target"]:
-                d_bs[key].append(line[i_key])
+                d_bs[key].append(line[idx_key])
+            # other keys include:  "Loc. X/Y/Z", "EMG Data 1/2", "EMG Peak-to-Peak 1" etc...
             else:
-                if ";" in line[i_key]:
-                    d_bs[key].append(np.array(line[i_key].split(';')).astype(float))
-
+                # each sample of EMG data is separated by a semicolon
+                if ";" in line[idx_key]:
+                    d_bs[key].append(np.array(line[idx_key].split(';')).astype(float))
+                # other non-EMG-data (numeric) entries
                 else:
-                    if line[i_key] != "(null)":
-                        d_bs[key].append(float(line[i_key]))
+                    # if key was selected for export but no corresponding data could be collected, "(null)" is entered
+                    if line[idx_key] != "(null)":
+                        d_bs[key].append(float(line[idx_key]))
                     else:
                         d_bs[key].append(float(-1))
 
+            # assemble coil configuration matrix
             if key in ["Loc. X"]:
-                m_nnav[0, 3, i_loc] = float(d_bs[key][-1])
+                m_nnav[0, 3, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["Loc. Y"]:
-                m_nnav[1, 3, i_loc] = float(d_bs[key][-1])
+                m_nnav[1, 3, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["Loc. Z"]:
-                m_nnav[2, 3, i_loc] = float(d_bs[key][-1])
+                m_nnav[2, 3, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m0n0"]:
-                m_nnav[0, 0, i_loc] = float(d_bs[key][-1])
+                m_nnav[0, 0, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m0n1"]:
-                m_nnav[1, 0, i_loc] = float(d_bs[key][-1])
+                m_nnav[1, 0, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m0n2"]:
-                m_nnav[2, 0, i_loc] = float(d_bs[key][-1])
+                m_nnav[2, 0, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m1n0"]:
-                m_nnav[0, 1, i_loc] = float(d_bs[key][-1])
+                m_nnav[0, 1, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m1n1"]:
-                m_nnav[1, 1, i_loc] = float(d_bs[key][-1])
+                m_nnav[1, 1, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m1n2"]:
-                m_nnav[2, 1, i_loc] = float(d_bs[key][-1])
+                m_nnav[2, 1, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m2n0"]:
-                m_nnav[0, 2, i_loc] = float(d_bs[key][-1])
+                m_nnav[0, 2, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m2n1"]:
-                m_nnav[1, 2, i_loc] = float(d_bs[key][-1])
+                m_nnav[1, 2, idx_coil_sample] = float(d_bs[key][-1])
             elif key in ["m2n2"]:
-                m_nnav[2, 2, i_loc] = float(d_bs[key][-1])
+                m_nnav[2, 2, idx_coil_sample] = float(d_bs[key][-1])
 
-            m_nnav[3, 3, i_loc] = 1
+            m_nnav[3, 3, idx_coil_sample] = 1
 
     # transform from brainsight to simnibs space
     ############################################
@@ -427,8 +439,10 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
     if verbose:
         print(f"Creating dictionary containing stimulation and physiological data")
 
-    current_scaling_factor = 1.43
+    current_scaling_factor = 1.43 if "current_scaling_factor" not in subject.exp[exp_idx].keys()\
+        else subject.exp[exp_idx]["current_scaling_factor"]
 
+    # create output directory (which is going to be persisted to experiment.hdf5)
     d = dict()
     d['coil_0'] = []
     d['coil_1'] = []
@@ -447,13 +461,18 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
         d['coil_mean'].append(np.nanmean(np.stack((d['coil_0'][-1],
                                                    d['coil_1'][-1]), axis=2), axis=2))
         d['number'].append(d_bs['Index'][i])
-        d['condition'].append(d_bs['Sample Name'][i])
+        # Sample Name is unique for each pulse, so no conditions will be detected
+        #d['condition'].append(d_bs['Sample Name'][i])
+        # Assoc. Target will assignt he same number to pulses that had the same target/marker
+        # (despite minor deviations in the true location of the coil after the pulse)
+        d['condition'].append(d_bs['Assoc. Target'][i])
         d['current'].append(1 * current_scaling_factor)
         d['date'].append(d_bs["Date"][i])
         d['time'].append(d_bs["Time"][i])
         d['coil_sn'].append(os.path.split(fn_coil)[1])
         d['patient_id'].append(subject.id)
 
+        # add remaining data to keys that have not aleady been added or that should not not be added
         for key in d_bs.keys():
             if key not in ['Sample Name', 'Session Name', 'Index',
                            'Loc. X', 'Loc. Y', 'Loc. Z',
@@ -530,10 +549,7 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
     if coil_outlier_corr_cond:
         if verbose:
             print("Removing coil position outliers")
-        d = pynibs.coil_outlier_correction_cond(exp=d,
-                                         outlier_angle=5.,
-                                         outlier_loc=3.,
-                                         fn_exp_out=fn_exp_hdf5)
+        d = pynibs.coil_outlier_correction_cond(exp=d, outlier_angle=5., outlier_loc=3., fn_exp_out=fn_exp_hdf5)
 
     # perform coil <-> head distance correction
     ###########################################
@@ -541,9 +557,49 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
         if verbose:
             print("Performing coil <-> head distance correction")
         d = pynibs.coil_distance_correction(exp=d,
-                                     fn_geo_hdf5=fn_mesh_hdf5,
-                                     remove_coil_skin_distance_outlier=remove_coil_skin_distance_outlier,
-                                     fn_plot=os.path.split(fn_exp_hdf5)[0])
+                                            fn_geo_hdf5=fn_mesh_hdf5,
+                                            remove_coil_skin_distance_outlier=remove_coil_skin_distance_outlier,
+                                            fn_plot=os.path.split(fn_exp_hdf5)[0])
+
+    d_avg = dict()
+    for key in d.keys():
+        d_avg[key] = []
+
+    # TODO: This only meaningfully works in conjunction with 'coil_distance_correction', which averages the
+    #       coil position within a condition. For each condition, we wil then have 1) and average coil position,
+    #       2) an average EMG sample, 3) only 1 full dataset (all but one samples are discarded after averaging)
+    if average_emg_data_from_same_condition:
+        if verbose:
+            print("Averaging samples from the same condition")
+
+        conditions_per_sample = np.array(d['condition'])
+        conditions_unique = np.unique(conditions_per_sample)
+
+        # Create new, dedicated dictionary with all EMG data to:
+        # 1) collect all EG related keys of the main dictionary
+        # 2) convert the stored lists to numpy arrays (for list-based indexing)
+        all_emg_data = dict()
+        for c_idx, chan_name in enumerate(channels):
+            all_emg_data[f"mep_raw_data_{chan_name}"] = np.array(d[f"mep_raw_data_{chan_name}"])
+            all_emg_data[f"mep_filt_data_{chan_name}"] = np.array(d[f"mep_filt_data_{chan_name}"])
+            all_emg_data[f"p2p_brainsight_{chan_name}"] = np.array(d[f"p2p_brainsight_{chan_name}"])
+            all_emg_data[f"p2p_{chan_name}"] = np.array(d[f"p2p_{chan_name}"])
+
+        # create a single entry in the new result dictionary (d_avg) for each condition
+        for condition_id in conditions_unique:
+            condition_idcs = np.where(conditions_per_sample == condition_id)[0]
+
+            # just copy the first occurrence (within a condition) of non-emg related keys
+            if len(condition_idcs) > 0:
+                for key in d.keys():
+                    if key not in all_emg_data.keys():
+                        d_avg[key].append(d[key][condition_idcs[0]])
+
+                # average all emg-related keys of a condition
+                for key in all_emg_data.keys():
+                    d_avg[key].append(np.mean(all_emg_data[key][condition_idcs], axis=0))
+
+        d = d_avg
 
     # create dictionary of stimulation data
     #######################################
@@ -595,69 +651,4 @@ def merge_exp_data_brainsight(subject, exp_idx, mesh_idx, coil_outlier_corr_cond
         print(f"Saving experimental data to file: {fn_exp_hdf5}")
     df_stim_data.to_hdf(fn_exp_hdf5, "stim_data")
     df_phys_data_raw.to_hdf(fn_exp_hdf5, "phys_data/raw/EMG")
-    df_phys_data_postproc.to_hdf(fn_exp_hdf5, "phys_data/postproc/EMG")
-
-    # plot finally used mep data
-    ############################
-    ''' # done directly in 'calc_p2p' now to avoid code duplication
-    if plot:
-        if verbose:
-            print("Creating MEP plots ...")
-        sampling_rate = 1000 / d_bs["EMG Res."][0]
-
-        # Compute start and stop idx according to sampling rate
-        start_mep = int((18 / 1000.) * sampling_rate)
-        end_mep = int((35 / 1000.) * sampling_rate)
-
-        # compute tms pulse idx in samplerate space
-        tms_pulse_sample = int(d[f"Offset"][0] * sampling_rate)
-
-        for i_mep in tqdm(range(len(d["coil_0"]))):
-            for i_channel, channel in enumerate(channels):
-                sweep = d[f"mep_raw_data_{channel}"][i_mep]
-                sweep_filt = d[f"mep_filt_data_{channel}"][i_mep]
-
-                # get index for begin of mep search window
-                # index_max_begin = np.argmin(sweep) + start_mep  # get TMS impulse # int(0.221 / 0.4 * sweep.size)
-                # beginning of mep search window
-                srch_win_start = tms_pulse_sample + start_mep  # get TMS impulse # in
-
-                if srch_win_start >= sweep_filt.size:
-                    srch_win_start = sweep_filt.size - 1
-
-                # index_max_end = sweep_filt.size  # int(0.234 / 0.4 * sweep.size) + 1
-                srch_win_end = srch_win_start + end_mep
-
-                fn_channel = os.path.join(os.path.split(subject.exp[exp_idx]["fn_data"][0][0])[0], "plots",
-                                          str(channel))
-
-                if not os.path.exists(fn_channel):
-                    os.makedirs(fn_channel)
-
-                fn_plot = os.path.join(fn_channel, f"mep_{i_mep:04}")
-                t = np.arange(len(sweep)) / sampling_rate
-                sweep_min_idx = np.argmin(sweep_filt[srch_win_start:srch_win_end]) + srch_win_start
-                sweep_max_idx = np.argmax(sweep_filt[srch_win_start:srch_win_end]) + srch_win_start
-
-                plt.figure(figsize=[4.07, 3.52])
-                plt.plot(t, sweep)
-                plt.plot(t, sweep_filt)
-                plt.scatter(t[sweep_min_idx], sweep_filt[sweep_min_idx], 15, color="r")
-                plt.scatter(t[sweep_max_idx], sweep_filt[sweep_max_idx], 15, color="r")
-                plt.plot(t, np.ones(len(t)) * sweep_filt[sweep_min_idx], linestyle="--", color="r", linewidth=1)
-                plt.plot(t, np.ones(len(t)) * sweep_filt[sweep_max_idx], linestyle="--", color="r", linewidth=1)
-                plt.grid()
-                plt.legend(["raw", "filtered", "p2p"], loc='upper right')
-
-                plt.xlim([np.max((d[f"Offset"][0] - 0.01, np.min(t))),
-                          np.min((t[tms_pulse_sample + end_mep] + 0.1, np.max(t)))])
-                plt.ylim([-1.1 * np.abs(sweep_filt[sweep_min_idx]), 1.1 * np.abs(sweep_filt[sweep_max_idx])])
-
-                plt.xlabel("time in s", fontsize=11)
-                plt.ylabel("MEP in mV", fontsize=11)
-                plt.tight_layout()
-
-                plt.savefig(fn_plot, dpi=200, transparent=True)
-                plt.close()
-
-    '''
+    df_phys_data_postproc.to_hdf(fn_exp_hdf5, "phys_data/postproc/EMG")
\ No newline at end of file
