@@ -258,10 +258,10 @@ def match_tm_to_im(tm_array, im_array, tms_time_arr, tms_cur_arr):
 
     Parameters
     ----------
-    tm_array : ndarray of float [Nx4x4]
-        Mean-value of Nx4x4 coil matrices
-    im_array : ndarray of float [Mx4x4]
-        Instrument-marker-matrices
+    tm_array : np.ndarray of float
+        (N, 4, 4) Mean-value of Nx4x4 coil matrices.
+    im_array : np.ndarray of float
+        (M, 4, 4) Instrument-marker-matrices.
 
     Returns
     -------
@@ -369,16 +369,16 @@ def get_marker(im_path, markertype):
     Parameters
     ----------
     im_path : str or list of str
-        Path to instrument-marker-file
+        Path to instrument-marker-file.
 
     Returns
     -------
-    im_array : np.array of float [Mx4x4]
-        Instrument-marker-matrices
+    im_array : np.array of float
+        (M, 4, 4) Instrument-marker-matrices.
     im_cond_lst : list of str
-        Labels of the instrument-marker-conditions
+        Labels of the instrument-marker-conditions.
     im_marker_times : list of float
-        Onset times
+        Onset times.
     """
     if isinstance(im_path, str):
         return get_single_marker_file(im_path, markertype)
@@ -432,14 +432,14 @@ def get_single_marker_file(im_path, markertype):
         Parameters
         ----------
         im_path : str or list of str
-            Path to instrument-marker-file
+            Path to instrument-marker-file.
 
         Returns
         -------
-        im_array : np.array of float [Mx4x4]
-            Instrument-marker-matrices
+        im_array : np.array of float
+            (M, 4, 4) Instrument-marker-matrices.
         im_cond_lst : list of str
-            Labels of the instrument-marker-conditions
+            Labels of the instrument-marker-conditions.
         """
     im_array = np.empty([0, 4, 4])
     marker_descriptions = []
@@ -482,16 +482,16 @@ def read_triggermarker_localite(fn_xml):
     Returns
     -------
     list of
-        m_nnav : ndarray of float [4x4xN]
-            Neuronavigation coordinates of N stimuli (4x4 matrices)
-        didt : ndarray of float [N]
-            Rate of change of coil current in (A/us)
-        stim_int : ndarray of float [N]
-            Stimulator intensity in (%)
-        descr : list of str [N]
-            Labels of the instrument-marker-conditions
-        rec_time : list of str [N]
-            Recording time in ms
+        m_nnav : np.ndarray of float
+            (M, 4, 4) Neuronavigation coordinates of N stimuli (4x4 matrices).
+        didt : np.ndarray of float
+            (N) Rate of change of coil current in (A/us).
+        stim_int : np.ndarray of float
+            (N) Stimulator intensity in (%).
+        descr : list of str
+            (N) Labels of the instrument-marker-conditions.
+        rec_time : list of str
+            (N) Recording time in ms.
     """
 
     m_nnav = np.empty([4, 4, 0])
@@ -545,34 +545,34 @@ def merge_exp_data_localite(subject, coil_outlier_corr_cond, remove_coil_skin_di
 
     Parameters
     ----------
-    subject : pyfempp.subject
-        Subject object
+    subject : pynibs.subject.Subject
+        Subject object.
     exp_idx : str
-        Experiment ID
+        Experiment ID.
     mesh_idx : str
-        Mesh ID
+        Mesh ID.
     coil_outlier_corr_cond : bool
-        Correct outlier of coil position and orientation (+-2 mm, +-3 deg) in case of conditions
+        Correct outlier of coil position and orientation (+-2 mm, +-3 deg) in case of conditions.
     remove_coil_skin_distance_outlier : bool
-        Remove outlier of coil position lying too far away from the skin surface (+- 5 mm)
+        Remove outlier of coil position lying too far away from the skin surface (+- 5 mm).
     coil_distance_corr : bool
-        Perform coil <-> head distance correction (coil is moved towards head surface until coil touches scalp)
+        Perform coil <-> head distance correction (coil is moved towards head surface until coil touches scalp).
     drop_mep_idx : List of int or None
         Which MEPs to remove before matching.
-    mep_onsets : List of int or None (Default: None)
-        If there are multiple .cfs per TMS Navigator sessions, onsets in [ms] of .cfs. E.g.: [0, 71186].
+    mep_onsets : List of int or None, default: None
+        If there are multiple .cfs per TMS Navigator sessions, onsets in [ms] of .cfs. E.g.: ``[0, 71186]``.
     cfs_data_column : int or list of int
         Column(s) of dataset in .cfs file.
     channels : list of str
-        List of channel names
+        List of channel names.
     verbose : bool
-        Plot output messages
+        Plot output messages.
     plot : bool, optional, default: False
         Plot MEPs and p2p evaluation
     start_mep : float, optional, default: 18
-        Start of time frame after TMS pulse where p2p value is evaluated (in ms)
+        Start of time frame after TMS pulse where p2p value is evaluated (in ms).
     end_mep : float, optional, default: 35
-        End of time frame after TMS pulse where p2p value is evaluated (in ms)
+        End of time frame after TMS pulse where p2p value is evaluated (in ms).
     """
     if channels is None:
         channels = ["channel_0"]
@@ -822,26 +822,26 @@ def merge_exp_data_rt(subject, coil_outlier_corr_cond, remove_coil_skin_distance
 
     Parameters
     ----------
-    subject : pynibs.subject
-        Subject object
+    subject : pynibs.subject.Subject
+        Subject object.
     exp_idx : str
-        Experiment ID
+        Experiment ID.
     mesh_idx : str
-        Mesh ID
+        Mesh ID.
     coil_outlier_corr_cond : bool
-        Correct outlier of coil position and orientation (+-2 mm, +-3 deg) in case of conditions
+        Correct outlier of coil position and orientation (+-2 mm, +-3 deg) in case of conditions.
     cond : string
-        Which condition to analyse
+        Which condition to analyse.
     remove_coil_skin_distance_outlier : bool
-        Remove outlier of coil position lying too far away from the skin surface (+- 5 mm)
+        Remove outlier of coil position lying too far away from the skin surface (+- 5 mm).
     coil_distance_corr : bool
-        Perform coil <-> head distance correction (coil is moved towards head surface until coil touches scalp)
+        Perform coil <-> head distance correction (coil is moved towards head surface until coil touches scalp).
     drop_trial_idx : List of int or None
         Which MEPs to remove before matching.
     verbose : bool
-        Plot output messages
+        Plot output messages.
     plot : bool, optional, default: False
-        Plot MEPs and p2p evaluation
+        Plot RTs and a running average over 10 trials.
     """
     behavior_paths_lst = subject.exp[exp_idx]['fn_data']
     tms_paths_lst = subject.exp[exp_idx]['fn_tms_nav']
@@ -962,10 +962,35 @@ def merge_exp_data_rt(subject, coil_outlier_corr_cond, remove_coil_skin_distance
                                                       remove_coil_skin_distance_outlier=remove_coil_skin_distance_outlier,
                                                       fn_plot=os.path.split(fn_exp_hdf5)[0])
 
-    # plot finally used mep data
+    # plot finally used rt data
     ############################
     if plot:
-        raise NotImplementedError("Plotting is not implemented yet.")
+        # assign data
+        x = results_dct['number']
+        y = results_dct['rt']
+
+        # calculate running average over 10 trials
+        avg_window = 10
+        average_y = []
+        for ind in range(len(y) - avg_window + 1):
+            average_y.append(np.mean(y[ind:ind + avg_window]))
+        # insert NaNs to match lengths
+        for ind in range(avg_window - 1):
+            average_y.insert(0, np.nan)
+
+        # create plot
+        plt.close()
+        plt.figure(num='RT-plot for subject ' + subject.id)
+        plt.scatter(x, y, s=10, label='reaction times in ms')
+        plt.plot(x, average_y, color='blue', linestyle='-', label='running average')
+        plt.xlabel('trial number')
+        plt.legend()
+        plt.title('development of the reaction times over the course of the trial')
+        fn_rt_plot = os.path.join(subject_obj.subject_folder, "exp", "model_TMS", "plot_RT.png")
+        plt.savefig(fn_rt_plot, dpi=600)
+        # plt.show()
+        plt.close()
+
 
     # Write experimental data to hdf5
     ###############################################
