@@ -1,11 +1,19 @@
-import os
-import shutil
-import subprocess
+"""
+This holds methods to interact with FreeSurfer ([1]_), for example to translate FreeSurfer files into Paraview readable
+.vtk files.
 
+References
+----------
+.. [1] Dale, A.M., Fischl, B., Sereno, M.I., 1999. Cortical surface-based analysis. I. Segmentation and surface
+   reconstruction. Neuroimage 9, 179-194.
+"""
+import os
 import h5py
+import shutil
 import meshio
 import nibabel
 import platform
+import subprocess
 import numpy as np
 import nibabel as nib
 import pynibs
@@ -13,23 +21,23 @@ import pynibs
 
 def read_curv_data(fname_curv, fname_inf, raw=False):
     """
-    Read curvature data provided by freesurfer and optionally process it.
+    Read curvature data provided by FreeSurfer with optional normalization.
 
     Parameters
     ----------
     fname_curv : str
-        Filename of the freesurfer curvature file (e.g. ?h.curv), contains curvature data in nodes
-        can be found in mri2mesh proband folder: /proband_ID/fs_ID/surf/?h.curv
+        Filename of the FreeSurfer curvature file (e.g. ?h.curv), contains curvature data in nodes
+        can be found in mri2mesh proband folder: ``proband_ID/fs_ID/surf/?h.curv``.
     fname_inf : str
-        Filename of inflated brain surface (e.g. ?h.inflated), contains points and connectivity data of surface
-        can be found in mri2mesh proband folder: /proband_ID/fs_ID/surf/?h.inflated
-    raw : boolean
-        Decide if raw-data is returned or if the data is normalized to -1 for neg. and +1 for pos. curvature
+        Filename of inflated brain surface (e.g. ``?h.inflated``), contains points and connectivity data of surface
+        can be found in mri2mesh proband folder: ``proband_ID/fs_ID/surf/?h.inflated``.
+    raw : bool
+        If raw-data is returned or if the data is normalized to -1 for neg. and +1 for pos. curvature.
 
     Returns
     -------
-    curv : nparray of float or int
-        Curvature data in element centers
+    curv : np.ndarray of float or int
+        Curvature data in element centers.
     """
 
     # load curvature data
@@ -41,7 +49,7 @@ def read_curv_data(fname_curv, fname_inf, raw=False):
     # interpolate point data to element centers
     curv = pynibs.data_nodes2elements(curv_points, con_inf)
 
-    # normalize curvature data (optional)
+    # normalize curvature data, optional
     if not raw:
         curv[curv > 0] = 1
         curv[curv < 0] = -1
@@ -54,36 +62,44 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
     """
     Creates a group average from scratch, based on one subject. This prevents for example the fsaverage problems of
     large elements at M1, etc.
+    This is an implemntation of [2]_ 'Creating a registration template from scratch (GW)'.
+
+    References
+    ----------
+    .. [2] https://surfer.nmr.mgh.harvard.edu/fswiki/SurfaceRegAndTemplates
 
     Parameters
     ----------
     subjects : list of str
-        List of freesurfer subjects names
+        List of FreeSurfer subjects names.
     subject_dir : str
-            temporary subject directory of freesurfer (symlinks of subjects will be generated in there and average
-            template will be temporarily stored before it is copied to average_dir)
-    average: string (Optional)
-        Which subject to base new average template on? Default: subjects[0]
-    hemi : string (Optional)
-        lh or rh
-    template : string <Default: mytemplate>
-        Basename of new template
-    steps : int <Default: 2>
-        Number of iterations
-    n_cpu : int <Default: 4>
-        How many cores for multithreading?
+        Temporary subject directory of FreeSurfer (symlinks of subjects will be generated in there and average
+        template will be temporarily stored before it is copied to ``average_dir``).
+    average: str, default: ``subjects[0]``
+        Which subject to base new average template on.
+    hemi : str, default: 'lh'
+        Which hemisphere: ``lh`` or ``rh``.
+
+        .. deprecated:: 0.0.1
+            Don't use any more.
+    template : str, default: 'mytemplate'
+        Basename of new template.
+    steps : int, default: 2
+        Number of iterations.
+    n_cpu : int, default: 4
+        How many cores for multithreading.
     average_dir : str
-        Path to directory where average template will be stored
-        (e.g.: probands/avg_template_15/mesh/0/fs_avg_template_15)
+        Path to directory where average template will be stored,
+        e.g. ``probands/avg_template_15/mesh/0/fs_avg_template_15``.
 
     Returns
     -------
     <File> : .tif file
-        SUBJECT_DIR/TEMPLATE*.tif, TEMPLATE0.tif based on AVERAGE, rest on all subjects
+        ``SUBJECT_DIR/TEMPLATE*.tif``, TEMPLATE0.tif based on AVERAGE, rest on all subjects.
     <File> : .myreg file
-        SUBJECT_DIR/SUBJECT*/surf/HEMI.sphere.myreg*
+        ``SUBJECT_DIR/SUBJECT*/surf/HEMI.sphere.myreg*``.
     <File> : .tif file
-        Subject wise sphere registration based on TEMPLATE*.tif
+        Subject wise sphere registration based on ``TEMPLATE*.tif``.
     """
     subject_names = [os.path.split(subjects[i])[1] for i in range(len(subjects))]
 
@@ -91,7 +107,7 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
     if not os.path.exists(subject_dir):
         print(('Creating $SUBJECTS_DIR in {}'.format(subject_dir)))
         os.makedirs(subject_dir)
-    print(('Set environment variable of freesurfer SUBJECTS_DIR={}'.format(subject_dir)))
+    print(('Set environment variable of FreeSurfer SUBJECTS_DIR={}'.format(subject_dir)))
     os.putenv('SUBJECTS_DIR', subject_dir)
 
     # make symlinks of subjects in $SUBJECT_DIR
@@ -100,36 +116,22 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
             print(('Generating symlink of subject {} in {}'.format(subject_names[i], subject_dir)))
             pynibs.bash('ln -s ' + subject + ' ' + subject_dir)
 
-    # if not subject_dir:
-    #     subject_dir = "/data/pt_01756/tmp/fs_ownav/"
-    # if not subject_dir.endswith('/'):
-    #     subject_dir += '/'
-    # if template.endswith(".tif"):
-    #     template = template[:-4]
-
     os.chdir(subject_dir)
 
-    # if not subjects:
-    #     subjects = ["fs_15484.08",
-    #                 "fs_08950.3f"]  # , "fs_09440.22", "fs_14102.d1", "fs_15891.1c", "fs_19925.e6", "fs_22793.3b",
-    #                    "fs_24417.b5", "fs_26824.0f", "fs_28027.87", "fs_28093.e5", "fs_29721.12", "fs_29965.48",
-    #                    "fs_30252.51", "fs_31715.c2", "fs_18080.2d"]
-
     if not average:
         average = subjects[0]
 
     print("Creating initial template")
     "mris_make_template lh sphere fs_15484.08 ./templates.tif"
 
-    command = "FREESURFER mris_make_template {} sphere {} lh{}0.tif".format("lh", os.path.split(average)[1], template)
+    command = "mris_make_template {} sphere {} lh{}0.tif".format("lh", os.path.split(average)[1], template)
     os.system(command)
-    command = "FREESURFER mris_make_template {} sphere {} rh{}0.tif".format("rh", os.path.split(average)[1], template)
+    command = "mris_make_template {} sphere {} rh{}0.tif".format("rh", os.path.split(average)[1], template)
     os.system(command)
-    # print command
 
     processes = set()
     for i in range(steps):
-        command = "FREESURFER mris_register"
+        command = "mris_register"
         print("\n\nStep {}".format(i))
         print("############################################################################################")
         # start all subjects registrations to .tif* in parallel
@@ -153,27 +155,18 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
                 os.wait()
                 processes.difference_update([p for p in processes if p.poll() is not None])
 
-            #        for subject in subjects:
-            #           print "        mris_register step {}{}".format(i, subject)
-            #         command = "FREESURFER mris_register {}/surf/{}.sphere {}{}.tif
-            #         {}.sphere.myreg{}".format(subject,hemi,template,i,hemi,i)
-            #       print command
-            #     os.system(command)
-        # needed? wait for all processes to finish
         while len(processes) > 0:
             os.wait()
             processes.difference_update([p for p in processes if p.poll() is not None])
 
         short_subjects = [os.path.split(subject)[1] for subject in subjects]
         print("        making template for step {}".format(i))
-        # command = "FREESURFER mris_make_template {} sphere {} rh{}0.tif".format("rh", os.path.split(average)[1],
-        #                                                                         template)
-        command = f"FREESURFER mris_make_template {'rh'} sphere.{template}{i} {' '.join(short_subjects)} " \
+        command = f"mris_make_template {'rh'} sphere.{template}{i} {' '.join(short_subjects)} " \
                   f"rh{template}{i + 1}.tif"
         print(command)
         os.system(command)
 
-        command = f"FREESURFER mris_make_template {'lh'} sphere.{template}{i} {' '.join(short_subjects)} " \
+        command = f"mris_make_template {'lh'} sphere.{template}{i} {' '.join(short_subjects)} " \
                   f"lh{template}{i + 1}.tif"
         print(command)
         os.system(command)
@@ -189,44 +182,39 @@ def make_group_average(subjects=None, subject_dir=None, average=None, hemi="lh",
 
 def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'):
     """
-    Generates the average template from a list of subjects using the freesurfer average.
+    Generates the average template from a list of subjects using the FreeSurfer average.
 
     Parameters
     ----------
     subjects : list of str
-        paths of subjects directories, where the freesurfer files are located
-        (e.g.: for simnibs mri2mesh .../fs_SUBJECT_ID)
+        Paths of subjects directories, where the FreeSurfer files are located,
+        e.g. for simnibs mri2mesh ``.../fs_SUBJECT_ID``.
     subject_dir : str
-        temporary subject directory of freesurfer (symlinks of subjects will be generated in there and average
-        template will be temporarily stored before it is copied to average_dir)
+        Temporary subject directory of FreeSurfer (symlinks of subjects will be generated in there and average
+        template will be temporarily stored before it is copied to ``average_dir``).
     average_dir : str
-        path to directory where average template will be stored
-        (e.g.: probands/avg_template_15/mesh/0/fs_avg_template_15
-    fn_reg : str <Default: sphere.reg --> ?h.sphere.reg>
-        Filename suffix of freesurfer registration file containing registration information to template
+        path to directory where average template will be stored,
+        e.g. probands/avg_template_15/mesh/0/fs_avg_template_15.
+    fn_reg : str, default: 'sphere.reg' --> ?h.sphere.reg>
+        Filename suffix of FreeSurfer registration file containing registration information to template.
 
     Returns
     -------
     <Files> : .tif and .reg files
-        Average template in average_dir and registered curvature files, ?h.sphere.reg in subjects/surf folders
+        Average template in average_dir and registered curvature files, ``?h.sphere.reg`` in subjects/surf folders.
     """
 
     subject_names = [os.path.split(subjects[i])[1] for i in range(len(subjects))]
 
     # check for right operating system
     if platform.system() != 'Linux':
-        raise OSError('Average template can only be generated with linux because of FREESURFER requirements!')
-
-    # check if FREESURFER is installed
-    output = pynibs.bash('which FREESURFER')
-    if not output:
-        raise Exception('FREESURFER not installed! Please visit https://freesurfer.net/ for more information')
+        raise OSError('Average template can only be generated with linux because of FreeSurfer requirements.')
 
     # define $SUBJECT_DIR of FREESURFER
     if not os.path.exists(subject_dir):
         print(('Creating $SUBJECTS_DIR in {}'.format(subject_dir)))
         os.makedirs(subject_dir)
-    print(('Set environment variable of freesurfer SUBJECTS_DIR={}'.format(subject_dir)))
+    print(('Set environment variable of FreeSurfer SUBJECTS_DIR={}'.format(subject_dir)))
     os.putenv('SUBJECTS_DIR', subject_dir)
 
     # make symlinks of subjects in $SUBJECT_DIR
@@ -240,7 +228,7 @@ def make_average_subject(subjects, subject_dir, average_dir, fn_reg='sphere.reg'
         shutil.rmtree(f'{subject_dir}{os.sep}avg_template')
     print(('Generating average subject in {}/avg_template'.format(subject_dir)))
     print('==============================================')
-    command = 'FREESURFER make_average_subject ' \
+    command = 'make_average_subject ' \
               '--out avg_template ' \
               ' --subjects ' + " ".join(subject_names) + \
               ' --surf-reg ' + fn_reg
@@ -262,36 +250,36 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
     Parameters
     ----------
     fn_subject_obj : str
-        Filename of subject object .pkl file (incl. path)
-        (e.g.: .../probands/subjectID/subjectID.pkl)
+        Filename of subject object .hdf5 file (incl. path),
+        e.g. ``.../probands/subjectID/subjectID.hdf5``
     fn_average_obj : str
-        Filename of average template object .pkl file (incl. path)
-        (e.g.: .../probands/avg_template/avg_template.pkl)
+        Filename of average template object .pkl file (incl. path),
+        e.g. ``.../probands/avg_template/avg_template.hdf5``.
     hemisphere : str
-        Define hemisphere to work on ('lh' or 'rh' for left or right hemisphere, respectively)
+        Define hemisphere to work on (``'lh'`` or ``'rh'`` for left or right hemisphere, respectively).
     fn_in_hdf5_data : str
-        Filename of .hdf5 data input file containing the subject data
+        Filename of .hdf5 data input file containing the subject data.
     data_hdf5_path : str
-        Path in .hdf5 data file where data is stored (e.g. '/data/tris/')
+        Path in .hdf5 data file where data is stored (e.g. ``'/data/tris/'``).
     data_label : str or list of str
-        Label of datasets contained in hdf5 input file to map
+        Label of datasets contained in hdf5 input file to map.
     fn_out_hdf5_geo : str
-        Filename of .hdf5 geo output file containing the geometry information
+        Filename of .hdf5 geo output file containing the geometry information.
     fn_out_hdf5_data : str
-        Filename of .hdf5 data output file containing the mapped data
+        Filename of .hdf5 data output file containing the mapped data.
     mesh_idx : int
-        Index of mesh used in the simulations
+        Index of mesh used in the simulations.
     roi_idx : int
-        Index of region of interest used in the simulations
-    subject_data_in_center : boolean
-        Specify if the data is given in the center of the triangles or in the nodes (Default = True)
+        Index of region of interest used in the simulations.
+    subject_data_in_center : bool, default: True
+        Specify if the data is given in the center of the triangles or in the nodes.
     data_substitute : float
         Data substitute with this number for all points outside the ROI mask
-    verbose : boolean
+    verbose : bool
         Verbose output (Default: True)
-    replace : boolean
+    replace : bool
         Replace output files (Default: True)
-    reg_fn : string
+    reg_fn : str
         Sphere.reg fn
 
     Returns
@@ -299,8 +287,8 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
     <Files> : .hdf5 files
         Geometry and corresponding data files to plot with Paraview:
 
-        - fn_out_hdf5_geo.hdf5: geometry file containing the geometry information of the average template
-        - fn_out_hdf5_data.hdf5: geometry file containing the data
+        * fn_out_hdf5_geo.hdf5: geometry file containing the geometry information of the average template
+        * fn_out_hdf5_data.hdf5: geometry file containing the data
     """
 
     subject = pynibs.load_subject(fname=fn_subject_obj)
@@ -391,21 +379,15 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
 
         del data_sub_nodes
 
-        # # remove NaNs from data because freesurfer can not handle it
-        # if verbose:
-        #     print("> Removing NaN from datasets and replacing it with data_substitute")
-        # for i in range(len(data_sub_nodes_mapped)):
-        #     data_sub_nodes_mapped[i][np.isnan(data_sub_nodes_mapped[i])] = data_substitute
-
         # Saving mapped subject data in nodes to .curv format for freesurfer (subject space)
         if verbose:
-            print('> Saving mapped subject data in nodes in .curv format for freesurfer')
+            print('> Saving mapped subject data in nodes in .curv format for FreeSurfer')
         for i in range(len(data_label)):
             nibabel.freesurfer.io.write_morph_data(fn_data_sub_nodes_mapped_curv[i], data_sub_nodes_mapped[i])
 
         # Set $SUBJECT_DIR of freesurfer
         if verbose:
-            print(('> Set environment variable of freesurfer SUBJECTS_DIR={}'.format(subject_dir)))
+            print(('> Set environment variable of FreeSurfer SUBJECTS_DIR={}'.format(subject_dir)))
         os.putenv('SUBJECTS_DIR', os.path.split(subject_dir)[0])
 
         # Make temporary symlink of average_dir in subject_dir
@@ -416,11 +398,11 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
                 os.system('rm -f ' + fn_avg_template_symlink_dest)
             os.system('ln -s ' + average_dir + ' ' + fn_avg_template_symlink_dest)
 
-        # map data on it using freesurfer
+        # map data on it using FreeSurfer
         if verbose:
-            print('> Mapping subject data to average template using freesurfer mri_surf2surf')
+            print('> Mapping subject data to average template using FreeSurfer mri_surf2surf')
         for i in range(len(data_label)):
-            fs_cmd = "FREESURFER mri_surf2surf --srcsubject " + os.path.split(subject_dir)[1] + \
+            fs_cmd = "mri_surf2surf --srcsubject " + os.path.split(subject_dir)[1] + \
                      " --trgsubject " + os.path.split(average_dir)[1] + \
                      " --srcsurfval " + fn_data_sub_nodes_mapped_curv[i] + \
                      " --trgsurfval " + fn_data_sub_nodes_mapped_avg_curv[i] + \
@@ -463,34 +445,35 @@ def data_sub2avg(fn_subject_obj, fn_average_obj, hemisphere, fn_in_hdf5_data, da
                                 replace=True)
 
     if verbose:
-        print('> Done!\n')
+        print('> Done.\n')
 
 
 def freesurfer2vtk(in_fns, out_folder, hem='lh', surf='pial', prefix=None, fs_subject='fsaverage',
                    fs_subjects_dir=None):
-    f"""
-    Transform multiple freesurfer .mgh files into one .vtk file.
+    """
+    Transform multiple FreeSurfer .mgh files into one .vtk file.
     This can be read with Paraview and others.
 
     Parameters
     ----------
-    in_fns : list of str or list
+    in_fns : list of str or str
+        Input filenames.
     out_folder : str
         Output folder.
     hem : str, default: 'lh'
-        Which hemisphere: 'lh' or 'rh'
+        Which hemisphere: ``'lh'`` or ``'rh'``.
     surf : str, default:  'pial'
-        Which freesurfer surface: 'pial', 'inflated', ...
+        Which FreeSurfer surface: ``'pial'``, ``'inflated'``, ...
     prefix : str, optional
-        Prefix to add to each filename
-    fs_subject : string, default: 'fsaverage'
-        FreeSurfer subject 
-    fs_subjects_dir : string, optional
-        FreeSurfer subjects directory. If not provided, read from environment
+        Prefix to add to each filename.
+    fs_subject : str, default: 'fsaverage'
+        FreeSurfer subject.
+    fs_subjects_dir : str, optional
+        FreeSurfer subjects directory. If not provided, read from environment.
 
     Returns
     -------
-    out_folder/{prefix}_{hem}_{surf}.vtk : File
+    <File>: out_folder/{prefix}_{hem}_{surf}.vtk
         One .vtk file with data as overlays from all .mgh files provided
     """
     if not isinstance(in_fns, list):
@@ -512,7 +495,7 @@ def freesurfer2vtk(in_fns, out_folder, hem='lh', surf='pial', prefix=None, fs_su
     for fn in in_fns:
         point_data[os.path.split(fn)[1].replace('.mgh', '')] = nib.load(fn).get_fdata()
 
-    # read freesurfer surface mesh
+    # read FreeSurfer surface mesh
     coords, faces = nib.freesurfer.read_geometry(f"{fs_subjects_dir}/{fs_subject}/surf/{hem}.{surf}")
 
     # write all dataarrays + mesh geometry into one .vtk
