@@ -112,25 +112,25 @@ class TetrahedraLinear:
 
         Parameters
         ----------
-        phi : np.ndarray of float [N_nodes x 1]
-            Scalar electric potential given in the nodes of the mesh
-        dAdt : np.ndarray of float [N_nodes x 3]
-            Magnetic vector potential given in the nodes of the mesh
-        roi : object instance
-            RegionOfInterestSurface object class instance
-        subject : Subject object
-            Subject object loaded from .hdf5 file
-        verbose : boolean
-            Print information to stdout
+        phi : np.ndarray of float
+            (N_nodes, 1) Scalar electric potential given in the nodes of the mesh.
+        dAdt : np.ndarray of float
+            (N_nodes, 3) Magnetic vector potential given in the nodes of the mesh.
+        roi : pynibs.mesh.mesh_struct.ROI
+            RegionOfInterestSurface object class instance.
+        subject : pynibs.subject.Subject
+            Subject object loaded from .hdf5 file.
+        verbose : bool
+            Print information to stdout.
         mesh_idx : int
-            Mesh index
+            Mesh index.
 
         Returns
         -------
-        E_normal : np.ndarray of float [N_points x 3]
-            Normal vector of electric field on GM-WM surface
-        E_tangential : np.ndarray of float [N_points x 3]
-            Tangential vector of electric field on GM-WM surface
+        E_normal : np.ndarray of float
+            (N_points, 3) Normal vector of electric field on GM-WM surface.
+        E_tangential : np.ndarray of float
+            (N_points, 3) Tangential vector of electric field on GM-WM surface.
         """
         import tempfile
         import simnibs.msh.mesh_io as mesh_io
@@ -270,25 +270,25 @@ class TetrahedraLinear:
 
         Parameters
         ----------
-        phi : np.ndarray of float [N_nodes x 1]
-            Scalar electric potential given in the nodes of the mesh
-        dAdt : np.ndarray of float [N_nodes x 3]
-            Magnetic vector potential given in the nodes of the mesh
-        roi : object instance
-            RegionOfInterestSurface object class instance
-        subject : Subject object
-            Subject object loaded from .hdf5 file
-        verbose : boolean
-            Print information to stdout
+        phi : np.ndarray of float
+            (N_nodes, 1) Scalar electric potential given in the nodes of the mesh.
+        dAdt : np.ndarray of float
+            (N_nodes, 1) Magnetic vector potential given in the nodes of the mesh.
+        roi : pynibs.mesh.mesh_struct.ROI
+            RegionOfInterestSurface object class instance.
+        subject : pynibs.subject.Subject
+            Subject object loaded from .hdf5 file.
+        verbose : bool
+            Print information to stdout.
         mesh_idx : int
-            Mesh index
+            Mesh index.
 
         Returns
         -------
-        E_normal : np.ndarray of float [N_points x 3]
-            Normal vector of electric field on GM-WM surface
-        E_tangential : np.ndarray of float [N_points x 3]
-            Tangential vector of electric field on GM-WM surface
+        E_normal : np.ndarray of float
+            (N_points, 3) Normal vector of electric field on GM-WM surface.
+        E_tangential : np.ndarray of float
+            (N_points, 3) Tangential vector of electric field on GM-WM surface.
         """
         import tempfile
         import simnibs.msh.mesh_io as mesh_io
@@ -435,14 +435,14 @@ class TetrahedraLinear:
 
         Parameters
         ----------
-        phi : np.ndarray of float [N_nodes x 1]
-            Scalar electric potential given in the nodes of the mesh
-        dAdt : np.ndarray of float [N_nodes x 3]
-            Magnetic vector potential given in the nodes of the mesh
-        roi : object instance
-            RegionOfInterestSurface object class instance
-        verbose : boolean
-            Print information to stdout
+        phi : np.ndarray of float
+            (N_nodes, 1) Scalar electric potential given in the nodes of the mesh.
+        dAdt : np.ndarray of float
+            (N_nodes, 3) Magnetic vector potential given in the nodes of the mesh.
+        roi : pynibs.mesh.mesh_struct.ORI
+            RegionOfInterestSurface object class instance.
+        verbose : bool
+            Print information to stdout.
         mode : str
             Select mode of output:
             - "components" : return x, y, and z component of tangential and normal components
@@ -450,10 +450,10 @@ class TetrahedraLinear:
 
         Returns
         -------
-        E_normal : np.ndarray of float [N_points x 3]
-            Normal vector of electric field on GM-WM surface
-        E_tangential : np.ndarray of float [N_points x 3]
-            Tangential vector of electric field on GM-WM surface
+        E_normal : np.ndarray of float
+            (N_nodes, 3) Normal vector of electric field on GM-WM surface.
+        E_tangential : np.ndarray of float
+            (N_nodes, 3) Tangential vector of electric field on GM-WM surface.
         """
         # check if dimension are fitting
         assert phi.shape[0] == dAdt.shape[0]
