@@ -144,7 +144,7 @@ def project_on_scalp_hdf5(coords, mesh, scalp_tag=1005):
     Find the node in the scalp closest to each coordinate
 
     Parameters
-    -------
+    ----------
     coords: nx3 np.ndarray
         Vectors to be transformed
     mesh: str or pynibs.TetrahedraLinear
@@ -153,7 +153,7 @@ def project_on_scalp_hdf5(coords, mesh, scalp_tag=1005):
         Tag in the mesh where the scalp is to be set. Default: 1005
 
     Returns
-    ------
+    -------
     points_closest: nx3 np.ndarray
         coordinates projected scalp (closest skin points)
     """
@@ -671,8 +671,8 @@ def midlayer_2_surf(midlayer_data, coords_target, coords_midlayer, midlayer_con=
     precise_map : bool, default=True
         If elements to nodes mapping is done, perform this precise and slow or not.
 
-    Returns:
-    --------
+    Returns
+    -------
     data_target : np.ndarray
         (n_nodes_target, 1) The data in nodes of the target surface.
     """
