@@ -19,29 +19,29 @@ def calc_tet_volume(points, abs=True):
     ----------
     points: np.ndarray
         shape: (n_tets,4,3)
-        [[[Ax, Ay, Az],
-          [Bx, By, Bz],
-          [Cx, Cy, Cz],
-          [Dx, Dy, Dy]],
 
-         [[Ax, Ay, Az],
-          [Bx, By, Bz],
-          [Cx, Cy, Cz],
-          [Dx, Dy, Dy]],
+        .. code-block:: sh
+           [[[Ax, Ay, Az],
+             [Bx, By, Bz],
+             [Cx, Cy, Cz],
+             [Dx, Dy, Dy]],
 
-        ...
-
-        ]
+            [[Ax, Ay, Az],
+             [Bx, By, Bz],
+             [Cx, Cy, Cz],
+             [Dx, Dy, Dy]],
+            ...
+           ]
 
     Returns
     -------
     volume: np.ndarray
-        shape: (n_tets)
+        shape: ``(n_tets)``
 
     Other Parameters
     ----------------
-    abs : bool
-        Return magnitude. Default: True.
+    abs : bool, default: true
+        Return magnitude
     """
     if points.ndim == 2:
         points = np.atleast_3d(points).reshape(1, 4, 3)
@@ -70,7 +70,7 @@ def calc_tri_surface(points):
 
     Returns
     -------
-    triange_area : np.ndarray
+    triangle_area : np.ndarray
     """
     a = np.linalg.norm(points[:, 0] - points[:, 1], axis=1)
     b = np.linalg.norm(points[:, 1] - points[:, 2], axis=1)
@@ -90,23 +90,31 @@ def get_sphere(mesh=None, mesh_fn=None, target=None, radius=None, roi_idx=None,
 
     Parameters
     ----------
-    mesh : pynibs.mesh.TetrahedraLinear, optional
+    mesh : pynibs.mesh.mesh_struct.TetrahedraLinear, optional
     mesh_fn : str, optional
-        Filename to SimNIBS .msh or pyNIBS .hdf5 mesh file
-    target : np.ndarray of float, optional
-        (3,) X, Y, Z coordinates of target
-    radius : float, optional
-        Sphere radius im mm
-    roi_idx : str or int
-        ROI name
-    elmtype : str, optional
-        Return triangles or tetrahedra in sphere around target. One of ('tris', 'tets')
+        Filename to SimNIBS .msh or pyNIBS .hdf5 mesh file.
+    target : np.ndarray of float or list of float
+        (3,) X, Y, Z coordinates of target.
+    radius : float
+        Sphere radius im mm.
+    roi_idx : str or int, optional
+        ROI name.
+    elmtype : str, default: 'tris'
+        Return triangles or tetrahedra in sphere around target. One of ('tris', 'tets').
 
     Returns
     -------
     elms_in_sphere : np.ndarray
         (n_elements): Indices of elements found in ROI
     """
+    # let's handle the input parameter combinations
+    assert target is not None
+    assert mesh is not None or mesh_fn is not None
+    if elmtype.lower().startswith('tri'):
+        elmtype = "tris"
+    elif elmtype.lower().startswith('tet'):
+        elmtype = "tets"
+
     if mesh_fn is not None:
         if mesh is not None:
             raise ValueError("Either provide mesh or mesh_fn")
@@ -114,16 +122,19 @@ def get_sphere(mesh=None, mesh_fn=None, target=None, radius=None, roi_idx=None,
             mesh = pynibs.load_mesh_hdf5(mesh_fn)
         elif mesh_fn.endswith('.msh'):
             mesh = pynibs.load_mesh_msh(mesh_fn)
+
     if roi is None and roi_idx is not None:
         if mesh_fn is None:
-            raise ValueError("Provide mesh_fn to load roi from")
+            raise ValueError("Provide mesh_fn to load roi from.")
         roi = pynibs.load_roi_surface_obj_from_hdf5(mesh_fn)[roi_idx]
+    if roi is None and roi_idx is not None or roi is not None and roi_idx is None:
+        raise ValueError("Provide either roi and roi_idx or none of them.")
     if elmtype == 'tris':
         return tris_in_sphere(mesh=mesh, target=target, radius=radius, roi=roi)
     elif elmtype == 'tets':
         return tets_in_sphere(mesh=mesh, target=target, radius=radius, roi=roi)
     else:
-        raise ValueError(f"Unknown elmtpye '{elmtype}'")
+        raise ValueError(f"Unknown elmtye '{elmtype}'")
 
 
 def tets_in_sphere(mesh, target, radius, roi):
@@ -134,6 +145,8 @@ def tets_in_sphere(mesh, target, radius, roi):
     If roi object / idx and mesh fn is provided, the roi is expected to have midlayer information and the roi
     geometry is used.
 
+    If radius is None or 0, the nearest element is returned.
+
     Parameters
     ----------
     mesh : pynibs.TetrahedraLinear, optional
@@ -177,14 +190,16 @@ def tris_in_sphere(mesh, target, radius, roi):
     If roi object / idx and mesh fn is provided, the roi is expected to have midlayer information and the roi
     geometry is used.
 
+    If radius is None or 0, the nearest element is returned.
+
     Parameters
     ----------
     mesh : pynibs.mesh.TetrahedraLinear, optional
-    target : np.ndarray of float, optional
+    target : np.ndarray of float or list of float
         (3,) X, Y, Z coordinates of target
-    radius : float, optional
+    radius : float
         Sphere radius im mm
-    roi : pynibs.mesh.ROI, optional
+    roi : pynibs.mesh.mesh_struct.ROI, optional
         ROI
 
     Returns
@@ -215,14 +230,14 @@ def sample_sphere(n_points, r):
     Parameters
     ----------
     n_points: int
-        Number of points to be spread, must be odd
+        Number of points to be spread, must be odd.
     r: float
-        Radius of sphere
+        Radius of sphere.
 
     Returns
     -------
-    points: np.ndarray of float [N x 3]
-        Evenly spread points in a unit sphere
+    points: np.ndarray of float
+        (N x 3), Evenly spread points in a unit sphere.
     """
 
     assert n_points % 2 == 1, "The number of points must be odd"
@@ -256,7 +271,7 @@ def get_indices_discontinuous_data(data, con, neighbor=False, deviation_factor=2
         Data array to analyze given in the element center
     con : np.ndarray of float [n_data, 3 or 4]
         Connectivity matrix
-    neighbor : boolean, optional, default=False
+    neighbor : bool, default: False
         Return also the element index of the "best" neighbor (w.r.t. median of data)
     deviation_factor : float
         Allows data deviation from 1/deviation_factor < data[i]/median < deviation_factor
@@ -381,14 +396,14 @@ def find_nearest(array, value):
     Parameters
     ----------
     array : np.ndarray of float
-        Monotonic increasing array
+        Monotonic increasing array.
     value : float
-        Target value the nearest neighbor index in "array" is computed for
+        Target value the nearest neighbor index in ``array`` is computed for.
 
     Returns
     -------
     idx : int
-        Index j such that "value" is between array[j] and array[j+1]
+        Index j such that "value" is between array[j] and array[j+1].
 
     """
     n = len(array)
@@ -424,9 +439,9 @@ def in_hull(points, hull):
     Parameters
     ----------
     points : np.ndarray
-        (N_points x 3) Set of floating point data to test whether they are lying inside the hull or not
-    hull : scipy.spatial.Delaunay  or np.ndarray
-         (M x K) Surface data
+        (N_points x 3) Set of floating point data to test whether they are lying inside the hull or not.
+    hull : scipy.spatial.Delaunay or np.ndarray
+         (M x K) Surface data.
 
     Returns
     -------
@@ -443,8 +458,8 @@ def in_hull(points, hull):
 def calc_tetrahedra_volume_cross(P1, P2, P3, P4):
     """
     Calculates volume of tetrahedra specified by the 4 points P1...P4
-    multiple tetrahedra can be defined by P1...P4 as 2-D np.arrays
-    using the cross and vector dot product
+    multiple tetrahedra can be defined by P1...P4 as 2-D np.ndarrays
+    using the cross and vector dot product.
 
     .. math::
         P1=\\begin{bmatrix}
@@ -655,11 +670,11 @@ def determine_e_midlayer(fn_e_results, fn_mesh_hdf5, subject, mesh_idx, roi_idx,
         Mesh index
     roi_idx : int
         ROI index
-    n_cpu : int, optional, default: 4
+    n_cpu : int, default: 4
         Number of parallel computations
-    midlayer_fun : str, optional, default: "simnibs"
+    midlayer_fun : str, default: "simnibs"
         Method to determine the midlayer e-fields ("pynibs" or "simnibs")
-    phi_scaling : float, optional, default: 1.0
+    phi_scaling : float, default: 1.0
         Scaling factor of scalar potential to change between "m" and "mm"
 
     Returns
@@ -738,7 +753,7 @@ def check_islands_for_single_elm(source_elm, connectivity=None, adjacency=None,
     adjacency : np.ndparray, optional
         Adjenceny matrix (n_elm, n_elm). Weights are supposed to be number of shared nodes.
         Computed from neighbors if not provided.
-    island_crit : int, optional, default: 'any'
+    island_crit : int, default: 'any'
         How many nodes to define islands?
         'any' -> Elements connected via a single node or single edge are defined as an island.
         'node' -> Elements connected via a single _node_ are defined as an island.
@@ -824,7 +839,7 @@ def find_islands(connectivity=None, adjacency=None, island_crit='any', verbose=F
     adjacency : np.ndparray, optional
         Adjenceny matrix (n_elm, n_elm). Weights are supposed to be number of shared nodes.
         Computed from neighbors if not provided.
-    island_crit : int, optional, default: 'any'
+    island_crit : int or str, default: 'any'
         How many nodes to define islands?
         'any' -> Elements connected via a single node or single edge are defined as an island.
         'node' -> Elements connected via a single _node_ are defined as an island.
@@ -904,11 +919,11 @@ def find_island_elms(connectivity=None, adjacency=None, verbose=False, island_cr
     adjacency : np.ndparray, optional
         Adjenceny matrix (n_elm, n_elm). Weights are supposed to be number of shared nodes.
         Computed from neighbors if not provided.
-    island_crit : int, optional, default: 'edge'
+    island_crit : int, default: 'edge'
         How many nodes to define islands?
         'node' -> Elements connected via a single _node_ are defined as an island.
         'edge' -> Elements connected via a single _edge_ are defined as an island.
-    decision : string, optional, default: cumulative
+    decision : str, default: cumulative
         'cumulative' -> Return all element indices that are not visited any times
         'smallest'   -> Return smallest island.
     verbose : bool, optional
@@ -958,3 +973,74 @@ def find_island_elms(connectivity=None, adjacency=None, verbose=False, island_cr
 
     elif decision == 'cumulative':
         return np.argwhere(counter_not_visited > 0)
+
+
+def cortical_depth(mesh_fn, geo_fn=None, write_xdmf=True, skin_surface_id=1005, verbose=False):
+    """
+    Compute skin-cortex-distance (SCD) for surface and volume data in ``mesh_fn``.
+
+    .. figure:: ../../doc/images/cortical_depth.png
+       :scale: 50 %
+       :alt: Visualized cortical depth.
+
+    Cortical depth computed against skin surface.
+
+    Parameters
+    ----------
+    mesh_fn : str
+        :py:class:`~pynibs.mesh.mesh_struct.TetrahedraLinear` mesh file.
+    geo_fn : str, optional
+        :py:class:`~pynibs.mesh.mesh_struct.TetrahedraLinear` mesh file with geometric data. If provided, geometric
+        information is read from here.
+    write_xdmf : bool, default: True
+        Write .xdmf or not.
+    skin_surface_id : int, default: 1005
+        Which tissue type nr to compute distance against.
+    verbose : bool, default: False
+        Print some verbosity information.
+
+    Returns
+    -------
+    <file> : .hdf5
+        ``mesh_fn`` or ``geo_fn`` with SCD information in ``/data/tris/Cortex_dist`` and ``/data/tets/Cortex_dist``.
+    <file> : .xdmf
+        Only if ``write_xdmf == True``.
+    """
+    if geo_fn is None:
+        geo_fn = mesh_fn
+    with h5py.File(mesh_fn, 'r') as f:
+        skin_tri_idx = f['/mesh/elm/tri_tissue_type'][:] == skin_surface_id
+        tri_nodes = f['/mesh/elm/triangle_number_list'][:][skin_tri_idx]
+
+    elms_with_island, counter_visited, counter_not_visited = pynibs.find_islands(connectivity=tri_nodes,
+                                                                                 verbose=True,
+                                                                                 island_crit='edge',
+                                                                                 largest=True)
+    hdf5 = pynibs.load_mesh_hdf5(mesh_fn)
+
+    with h5py.File(geo_fn, 'r') as geo:
+        # get indices for skin elements
+        skin_tri_idx = np.squeeze(np.argwhere((geo['mesh/elm/tri_tissue_type'][:] == skin_surface_id)))
+        skin_positions = hdf5.triangles_center[skin_tri_idx[counter_visited > 0]]
+
+    def fun(row):
+        return np.min(np.linalg.norm(row - skin_positions, axis=1))
+
+    if verbose:
+        print("Computing triangles")
+    distances_tri = np.apply_along_axis(fun, axis=1, arr=hdf5.triangles_center)
+    if verbose:
+        print("Computing tetrahedra")
+
+    distances_tets = np.apply_along_axis(fun, axis=1, arr=hdf5.tetrahedra_center)
+    with h5py.File(mesh_fn, 'a') as f:
+        try:
+            del f['data/tris/Cortex_dist']
+            del f['data/tets/Cortex_dist']
+        except KeyError:
+            pass
+        f.create_dataset(name='data/tris/Cortex_dist', data=distances_tri)
+        f.create_dataset(name='data/tets/Cortex_dist', data=distances_tets)
+
+    if write_xdmf:
+        pynibs.write_xdmf(overwrite_xdmf=True, hdf5_geo_fn=geo_fn, hdf5_fn=mesh_fn)
