@@ -176,7 +176,7 @@ def load_roi_surface_obj_from_hdf5(fname):
                         elements = simnibs.Elements(triangles=node_number_list)
                         surf = simnibs.Msh(nodes=nodes, elements=elements)
 
-                        layers = getattr(roi[roi_idx],data_label[j])
+                        layers = getattr(roi[roi_idx], data_label[j])
                         layers.append(CorticalLayer.init_from_surface(layer_id, surf))
                 elif type(f[f'roi_surface/{roi_idx}/{data_label[j]}'][()]) == np.ndarray and \
                         "S" in str(f[f'roi_surface/{roi_idx}/{data_label[j]}'].dtype) and \
@@ -620,27 +620,27 @@ class CorticalLayer:
     def __init__(self, create_key, id, volumetric_mesh=None, roi=None, depth=None, path=None, surface=None):
         """
         Constructor of the cortical layer class. Three optional ways of construction a CorticalLayer-instance
-            a) by providing a path to an already existing layer (simnibs.Msh)
-            b) by proving a simnibs.Msh of an already existing layer.
-            c) by providing the bounding box of the ROI in which the layer should be created.
+        a) by providing a path to an already existing layer (simnibs.Msh)
+        b) by proving a simnibs.Msh of an already existing layer.
+        c) by providing the bounding box of the ROI in which the layer should be created.
 
         Parameters
         ----------
         id : str
             Identifier of the layer.
-        volumetric_mesh : simnibs.Msh (optinal)
+        volumetric_mesh : simnibs.Msh, optional
             The tetrahedral volume mesh, in which the layer should be generated.
         roi : RegionOfInterestSurface instance
             RegionOfInterestSurface
-        depth: float (optional)
+        depth: float, optional
             Normalized distance of the layer from gray matter surface.
             Provide values in the open interval (0,1)
-        elements_per_square_mm : float (optional)
+        elements_per_square_mm : float, optional
             Number of triangles per mm^2 in the layer.
             Default = 3.
-        path : str (optional)
+        path : str, optional
             File path to a region of interest surfe (e.g. midlayer).
-        surface : simnibs.Msh (optional)
+        surface : simnibs.Msh, optional
             The surface representation of an already existing layer (e.g. midlayer).
         """
         # trying to mimic a private constructor here, from: https://stackoverflow.com/a/46459300
@@ -1000,17 +1000,17 @@ class CorticalLayer:
     def get_smoothed_normals(self):
         """
         Computed the smoothed normals of the surface representation of this layer.
+
         Note: For the later stages, we don't want a smoothed surface, but smooth
-              normals in order to maintain the location of the cells, but orient
-              them more smoothly. Therefore, we use smoothed normals, e.g. for the
-              computation of the theta angle, but do not smooth the entire layer
-              surface.
+        normals in order to maintain the location of the cells, but orient
+        them more smoothly. Therefore, we use smoothed normals, e.g. for the
+        computation of the theta angle, but do not smooth the entire layer
+        surface.
 
         Returns
-        --------
+        -------
         normals : np.ndarray
             The tetrahedral volume mesh, in which the layer should be generated.
-
         """
         return self.surface.triangle_normals(smooth=CorticalLayer.Settings.NUM_TRIANGLE_SMOOTHING_STEPS).value
 
@@ -1022,7 +1022,6 @@ class CorticalLayer:
         ----------
         fn : str
             Target file name of the surface-file of this layer.
-
         """
         self.surface.write(fn)
 
@@ -1525,22 +1524,22 @@ class RegionOfInterestSurface:
                                   ):
         """
         Create the cortical layering with the provided laminar depths.
+
         Defaults to the standard depths of the laminae in the neo-cortex from layer I to VI
-            from "Simulation of transcranial magnetic stimulation in head model with morphologically-realistic
-            cortical neurons", Aberra et al., https://doi.org/10.1016/j.brs.2019.10.002
+        from "Simulation of transcranial magnetic stimulation in head model with morphologically-realistic
+        cortical neurons", Aberra et al., https://doi.org/10.1016/j.brs.2019.10.002
 
         Parameters
         ----------
         head_model_mesh : simnibs.Msh
             The head model volume mesh.
             Inside the GM compartment of this mesh, the layering will be generated.
-        bbox : np.ndarray (optional)
+        bbox : np.ndarray, optional
             Bounding coordinates of the region of interest.
             Optional, if the mid-layer surface is already existing
             (and can thus be used to determine the bounding coordinates).
         laminae : typing.Dict[str,float] | typing.Tuple
             List of depths of the individual to-be created lamiae.
-
         """
         assert bbox is not None or self.node_coord_mid is not None, \
             "Neither mid-layer surface was initialized nor explicit bounding box was provided to guide layer generation!"
@@ -1562,22 +1561,23 @@ class RegionOfInterestSurface:
 
         Parameters
         ----------
-        msh : pynibs.TetrahedraLinear
-            Object instance of TetrahedraLinear class
+        msh : pynibs.mesh.mesh_struct.TetrahedraLinear
+            TetrahedraLinear object.
 
         Returns
         -------
-        RegionOfInterestSurface.tet_idx_tri_center_up : np.ndarray [N_points]
-            Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
-            upper surface are lying in
-        RegionOfInterestSurface.tet_idx_tri_center_mid : np.ndarray [N_points]
-            Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
-            middle surface are lying in
-        RegionOfInterestSurface.tet_idx_tri_center_low : np.ndarray [N_points]
-            Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles of the
-            lower surface are lying in
-        RegionOfInterestSurface.tet_idx_node_coord_mid : np.ndarray [N_tri]
-            Tetrahedra indices of TetrahedraLinear object instance where the nodes of the middle surface are lying in
+        RegionOfInterestSurface.tet_idx_tri_center_up : np.ndarray
+            (N_points) Tetrahedra indices of TetrahedraLinear object instance where the center points of the
+            triangles of the upper surface are lying in.
+        RegionOfInterestSurface.tet_idx_tri_center_mid : np.ndarray
+            (N_points) Tetrahedra indices of TetrahedraLinear object instance where the center points of the triangles
+            of the middle surface are lying in.
+        RegionOfInterestSurface.tet_idx_tri_center_low : np.ndarray
+            (N_points) Tetrahedra indices of TetrahedraLinear object instance where the center points of the
+            triangles of the lower surface are lying in.
+        RegionOfInterestSurface.tet_idx_node_coord_mid : np.ndarray
+            (N_tri) Tetrahedra indices of TetrahedraLinear object instance where the nodes of the middle
+            surface are lying in.
      """
         # determine tetrahedra indices of triangle center points of upper, middle and lower surface
         if self.tri_center_coord_low != [] and self.tri_center_coord_up != []:
@@ -1607,21 +1607,51 @@ class RegionOfInterestSurface:
                                                                     points=self.node_coord_mid,
                                                                     compute_baricentric=False)
 
-    def subsample(self, dist=10, fn_sphere=""):
+    def decimate(self, fraction=.075):
         """
-        Subsample ROI surface and return element indices
+        Subsample ROI surface based on a decimation factor and return element indices.
+        (no Freesurfer surfaces associated with the ROI surface required)
+
+        Parameters
+        ----------
+        fraction : float, default: .075
+            Multiplied by the total number of ROI elements determines
+            (approximately) the number of remaining ROI elements after decimation.
+
+        Returns
+        -------
+        ele_idx : np.ndarray of float
+            [approx. fraction * n_ele] Element indices of the subsampled surface; sorted.
+        """
+
+        mesh = trimesh.Trimesh(
+            vertices=self.node_coord_mid,
+            faces=self.node_number_list
+        )
+
+        pts, subsample_idcs = trimesh.sample.sample_surface_even(
+            mesh=mesh,
+            count=int(self.node_number_list.shape[0] * fraction)
+        )
+
+        return np.sort(subsample_idcs)
+
+    def subsample(self, dist=10, fn_sphere=None):
+        """
+        Subsample ROI surface based on a spacing and return element indices
+        (Freesurfer surfaces associatd with the ROI surface required)
 
         Parameters
         ----------
         dist : float
-            Distance in mm the subsampled points lie apart
+            Distance in mm the subsampled points lie apart.
         fn_sphere : str
-            Name of ?.sphere file (freesurfer)
+            Name of ?.sphere file (freesurfer).
 
         Returns
         -------
-        ele_idx : ndarray of float [n_ele]
-            Element indices of the subsampled surface
+        ele_idx : ndarray of float
+            (n_ele) Element indices of the subsampled surface.
         """
 
         # load sphere surface
@@ -1667,22 +1697,22 @@ class RegionOfInterestVolume:
     Attributes
     ----------
     node_coord : np.ndarray [N_points x 3]
-        Coordinates (x,y,z) of ROI tetrahedra nodes
+        Coordinates (x,y,z) of ROI tetrahedra nodes.
     tet_node_number_list : np.ndarray [N_tet_roi x 3]
-        Connectivity matrix of ROI tetrahedra
+        Connectivity matrix of ROI tetrahedra.
     tri_node_number_list : np.ndarray [N_tri_roi x 3]
-        Connectivity matrix of ROI tetrahedra
+        Connectivity matrix of ROI tetrahedra.
     tet_idx_node_coord : np.ndarray [N_points]
-        Tetrahedra indices of TetrahedraLinear object instance where the ROI nodes are lying in
-    tet_idx_tetrahedra_center : np.ndarray [N_tet_roi]
-        Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI tetrahedra are
-        lying in
-    tet_idx_triangle_center : np.ndarray [N_tri_roi]
-        Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI triangle are
-        lying in. If the ROI is directly generated from the msh instance using "make_roi_volume_from_msh", these
+        Tetrahedra indices of TetrahedraLinear object instance where the ROI nodes are lying in.
+    tet_idx_tetrahedra_center : np.ndarray
+        (N_tet_roi) Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI
+        tetrahedra are lying in.
+    tet_idx_triangle_center : np.ndarray
+        (N_tri_roi) Tetrahedra indices of TetrahedraLinear object instance where the center points of the ROI triangle
+        are lying in. If the ROI is directly generated from the msh instance using "make_roi_volume_from_msh", these
         indices are the triangle indices of the head mesh since the ROI mesh and the head mesh are overlapping. If
         the ROI mesh is not the same as the head mesh, the triangle center of the ROI mesh are always lying in a
-        tetrahedra of the head mesh (these indices are given in this case)
+        tetrahedra of the head mesh (these indices are given in this case).
     """
 
     def __init__(self):
@@ -1701,17 +1731,20 @@ class RegionOfInterestVolume:
 
         Parameters
         ----------
-        msh: pynibs.TetrahedraLinear
-            Mesh object instance of type TetrahedraLinear (see main.py)
+        msh: pynibs.mesh.mesh_struct.TetrahedraLinear
+            Mesh object instance of type TetrahedraLinear
         volume_type: str
             Type of ROI ('box' or 'sphere')
         x_roi: list of float
+
             - type = 'box': [Xmin, Xmax] (in mm), whole X range if empty [0,0] or None (left - right)
             - type = 'sphere': origin [x,y,z]
         y_roi: list of float
+
             - type = 'box': [Ymin, Ymax] (in mm), whole Y range if empty [0,0] or None (anterior - posterior)
             - type = 'sphere': radius (in mm)
         z_roi: list of float
+
             - type = 'box': [Zmin, Zmax] (in mm), whole Z range if empty [0,0] or None (inferior - superior)
             - type = 'sphere': None
 
