@@ -20,6 +20,7 @@ class Subject:
     -----
     **Initialization**
 
+    .. code-block:: python
         sub = pynibs.subject(subject_ID, mesh)
 
     **Parameters**
@@ -199,7 +200,7 @@ class Subject:
 
         Parameters
         ----------
-        mesh_dict : dictionary or list of dictionaries
+        mesh_dict : dict or list of dict
             Dictionary containing the mesh information
 
         Notes
@@ -222,7 +223,7 @@ class Subject:
 
         Parameters
         ----------
-        roi_dict : dict of dict or list of dictionaries
+        roi_dict : dict of dict or list of dict
             Dictionary containing the ROI information of the mesh with mesh_index [mesh_idx][roi_idx]
 
         Notes
@@ -245,7 +246,7 @@ class Subject:
 
         Parameters
         ----------
-        ps_dict : dictionary or list of dictionaries
+        ps_dict : dict or list of dict
             Dictionary containing plot settings of the subject
 
         Notes
@@ -268,7 +269,7 @@ class Subject:
 
         Parameters
         ----------
-        mri_dict : dictionary or list of dictionaries
+        mri_dict : dict or list of dict
             Dictionary containing the MRI information of the subject
 
         Notes
