@@ -34,17 +34,17 @@ def rescale_lambda_centerized(D, s, tsc=False):
 
     Parameters
     ----------
-    D : nparray of float [3 x 3]
-        Diffusion tensor
+    D : nparray of float
+        (3, 3) Diffusion tensor.
     s : float
-        Scale parameter [0 (iso) ... 0.5 (unaltered)... 1 (aniso)]
-    tsc : boolean
-        Tensor singularity correction
+        Scale parameter [0 (iso) ... 0.5 (unaltered)... 1 (aniso)].
+    tsc : bool
+        Tensor singularity correction.
 
     Returns
     -------
-    Ds : nparray of float [3 x 3]
-        Scaled diffusion tensor
+    Ds : np.ndarray of float
+        (3, 3) Scaled diffusion tensor.
     """
 
     # print("Tensor scaling factor: " + str(s))
@@ -123,17 +123,17 @@ def rescale_lambda_centerized_workhorse(D, s, tsc=False):
 
     Parameters
     ----------
-    D : ndarray of float [n x 9]
-        Diffusion tensor
+    D : ndarray of float
+        (n, 9) Diffusion tensor.
     s : float
-        Scale parameter [0 (iso) ... 0.5 (unaltered)... 1 (aniso)]
-    tsc : boolean
-        Tensor singularity correction
+        Scale parameter [0 (iso) ... 0.5 (unaltered)... 1 (aniso)].
+    tsc : bool
+        Tensor singularity correction.
 
     Returns
     -------
-    Ds : list of nparray of float [3 x 3]
-        Scaled diffusion tensor
+    Ds : list of nparray of float
+        (3, 3) Scaled diffusion tensor
     """
 
     Ds = np.empty(D.shape)
