@@ -118,20 +118,20 @@ def nrmsd(array, array_ref, error_norm="relative", x_axis=False):
     Parameters
     ----------
     array: np.ndarray
-        input data [ (x), y0, y1, y2 ... ]
+        input data [ (x), y0, y1, y2 ... ].
     array_ref: np.ndarray
-        reference data [ (x_ref), y0_ref, y1_ref, y2_ref ... ]
-        if array_ref is 1D, all sizes have to match
+        reference data [ (x_ref), y0_ref, y1_ref, y2_ref ... ].
+        if array_ref is 1D, all sizes have to match.
     error_norm: str, optional, default="relative"
-        Decide if error is determined "relative" or "absolute"
-    x_axis: boolean, optional, default=False
+        Decide if error is determined "relative" or "absolute".
+    x_axis: bool, default: False
         If True, the first column of array and array_ref is interpreted as the x-axis, where the data points are
         evaluated. If False, the data points are assumed to be at the same location.
 
     Returns
     -------
-    normalized_rms: np.ndarray of float [array.shape[1]]
-        Normalized root-mean-square deviation between the columns of array and array_ref
+    normalized_rms: np.ndarray of float
+        ([array.shape[1]]) Normalized root-mean-square deviation between the columns of array and array_ref.
     """
 
     n_points = array.shape[0]
