@@ -2,7 +2,11 @@ import gc
 import re
 import copy
 import trimesh
-import simnibs
+
+try:
+    import simnibs
+except (ModuleNotFoundError, ImportError):
+    print("SimNIBS not found. Some functions might not work.")
 import warnings
 import collections
 import numpy as np
@@ -26,8 +30,20 @@ def e_field_gradient_between_wm_gm(
     ----------
     roi_surf : simnibs.Msh
         The surface object representing the ROI.
-    volumetric_mesh: simnibs.Msh
-        The (volumetric) head mesh.
+    mesh : simnibs.Msh
+        SimNIBS headmesh.
+    gm_nodes : np.ndarray, (3, len(roi_surf.nodes))
+        For each node of 'roi_surf' (representing, e.g., a cortical layer) the corresonding point on the gray matter surface. 
+        -> as returned by 'precompute_geo_info_for_layer_field_interpolation'
+    wm_nodes : np.ndarray, (3, len(roi_surf.nodes))
+        For each node of 'roi_surf' (representing, e.g., a cortical layer) the corresonding point on the white matter surface. 
+        -> as returned by 'precompute_geo_info_for_layer_field_interpolation'
+    gm_center_distance : np.ndarray, (3, len(roi_surf.nodes))
+        The distance between the layer nodes and their corresponding point on the gray matter suface. 
+        -> as returned by 'precompute_geo_info_for_layer_field_interpolation'
+    wm_center_distance : np.ndarray, (3, len(roi_surf.nodes))
+        The distance between the layer nodes and their corresponding point on the white matter suface. 
+        -> as returned by 'precompute_geo_info_for_layer_field_interpolation'
 
     Returns
     -------
@@ -35,28 +51,26 @@ def e_field_gradient_between_wm_gm(
         The E-field gradient from the GM to the WM surface normalized to 1 mm with respect to the gray matter thickness.
         per ROI node
     """
-    #roi_nodes = roi_surf.nodes.node_coord
     roi_centers = roi_surf.elements_baricenters().value
 
     # concatenate the point lists to avoid re-building the (simnibs-internal) data structures
     # for interpolation upon consecutive calls
     to_be_interpolated = np.concatenate((gm_nodes, roi_centers, wm_nodes))
     try:
-        interpolated = mesh.field['magnE'].interpolate_scattered( to_be_interpolated )
+        interpolated = mesh.field['magnE'].interpolate_scattered(to_be_interpolated)
     except KeyError:
-        magE = mesh.field['E'].norm()
-        interpolated = magE.interpolate_scattered( to_be_interpolated )
+        e_mag = mesh.field['E'].norm()
+        interpolated = e_mag.interpolate_scattered(to_be_interpolated)
 
     num_nodes = gm_nodes.shape[0]
-    num_nodes_x2 = num_nodes*2
-    num_nodes_x3 = num_nodes*3
+    num_nodes_x2 = num_nodes * 2
+    num_nodes_x3 = num_nodes * 3
     e_field_gm = interpolated[:num_nodes]
     e_field_center = interpolated[num_nodes:num_nodes_x2]
     e_field_wm = interpolated[num_nodes_x2:num_nodes_x3]
 
     e_field_gradient = e_field_gm - e_field_wm
     gray_matter_thickness = gm_center_distance + wm_center_distance
-    #distance_offset = gm_distance / gray_matter_thickness      # = Midlayer offset, may be needed later (?)
     e_field_gradient_per_mm = e_field_gradient / gray_matter_thickness
     relative_e_field_gradient_per_mm = e_field_gradient_per_mm / e_field_center * 100
 
@@ -95,41 +109,44 @@ def e_field_angle_theta(surface, mesh):
 def precompute_geo_info_for_layer_field_interpolation(simnibs_mesh, roi):
     """
     Precomputes geometric properties of the corresponding GM and WM nodes
-    in the 'simnibs_mesh' of each node of each layer in 'roi'.
-    > The corresponding point on the GM and WM surface to each vertex of te ROI surface
+    in the 'simnibs_mesh' of each node of each layer in ``'roi'``.
+
+    * The corresponding point on the GM and WM surface to each vertex of te ROI surface
       are determined (by raycasting and nearest neighbour search as fallback)
       (interpolation will take place between these nodes)
-    > The nodes are moved inside the gray matter by 20 % of their total distance from the GM/WM
+    * The nodes are moved inside the gray matter by 20 % of their total distance from the GM/WM
       boundary to the midlayer.
-    > The distance of the relocated GM and WM nodes to the ROI nodes is determined (required to
+    * The distance of the relocated GM and WM nodes to the ROI nodes is determined (required to
       computed the gradient per mm).
 
-     Parameters
-     ----------
-     simnibs_mesh : simnibs.Msh
-         The head model volume mesh.
-     roi: pynibs.roi.RegionOfInterestSurface
-         The ROI object containing the layers.
+    Parameters
+    ----------
+    simnibs_mesh : simnibs.Msh
+        The head model volume mesh.
+    roi: pynibs.roi.RegionOfInterestSurface
+        The ROI object containing the layers.
 
-     Returns
-     -------
-     layer_gm_wm_info : dict[str, dict[str,np.ndarray]]
-        For each layer defined in the mesh (outer dict key: layer_id),
-        provide pre-computed geometrical information as a dictionary (outer dict value):
-        - key: gm_nodes
-          For each layer node, the corresponding point on the GM surface.
-        - key: wm_nodes
-          For each layer node, the corresponding point on the WM surface.
-        - key: gm_center_distance
-          The distance between the layer nodes and the corresponding points on the GM.
-        - key: wm_center_distance
-          The distance between the layer nodes and the corresponding points on the WM.
+    Returns
+    -------
+    layer_gm_wm_info : dict[str, dict[str,np.ndarray]]
+       For each layer defined in the mesh (outer dict key: layer_id),
+       provide pre-computed geometrical information as a dictionary (outer dict value):
+
+       * key: gm_nodes
+         For each layer node, the corresponding point on the GM surface.
+       * key: wm_nodes
+         For each layer node, the corresponding point on the WM surface.
+       * key: gm_center_distance
+         The distance between the layer nodes and the corresponding points on the GM.
+       * key: wm_center_distance
+         The distance between the layer nodes and the corresponding points on the WM.
     """
     WHITE_MATTER_SURFACE_LABEL = 1001
     GRAY_MATTER_SURFACE_LABEL = 1002
 
     import vtk
-    def make_vtkpolydata( pts, tris ):
+
+    def make_vtkpolydata(pts, tris):
         # prepare vertices
         pts_vtk = vtk.vtkPoints()
         pts_vtk.SetNumberOfPoints(pts.shape[0])
@@ -141,7 +158,7 @@ def precompute_geo_info_for_layer_field_interpolation(simnibs_mesh, roi):
         for tri in tris:
             tris_vtk.InsertNextCell(3)
             for v in tri:
-                tris_vtk.InsertCellPoint( v )
+                tris_vtk.InsertCellPoint(v)
 
         # prepare GM polygonal surface
         surf_vtk = vtk.vtkPolyData()
@@ -160,9 +177,9 @@ def precompute_geo_info_for_layer_field_interpolation(simnibs_mesh, roi):
 
     # for each point on the layer find corresponding points on the GM/WM surfaces
     # using the VTK implementation of Raycasting
-                                        # make zero indexed and shape (n,3) instead of (n,4), with -1 in the 4th column
-    gm_surf_vtk = make_vtkpolydata(gm_nodes, gm_tris[:,:3] - 1)
-    wm_surf_vtk = make_vtkpolydata(wm_nodes, wm_tris[:,:3] - 1)
+    # make zero indexed and shape (n,3) instead of (n,4), with -1 in the 4th column
+    gm_surf_vtk = make_vtkpolydata(gm_nodes, gm_tris[:, :3] - 1)
+    wm_surf_vtk = make_vtkpolydata(wm_nodes, wm_tris[:, :3] - 1)
 
     gm_intersector = vtk.vtkOBBTree()
     gm_intersector.SetDataSet(gm_surf_vtk)
@@ -173,28 +190,29 @@ def precompute_geo_info_for_layer_field_interpolation(simnibs_mesh, roi):
     wm_intersector.BuildLocator()
 
     intersectors = [gm_intersector, wm_intersector]
-    normal_sign  = [1,-1]
+    normal_sign = [1, -1]
 
     layer_gm_wm_info = dict()
     for layer_idx in range(len(roi.layers)):
         intersec_pts = [[], []]
-        layer_id      = roi.layers[layer_idx].id
-        layer_surf    = roi.layers[layer_idx].surface
+        layer_id = roi.layers[layer_idx].id
+        layer_surf = roi.layers[layer_idx].surface
         layer_centers = layer_surf.elements_baricenters().value
         layer_normals = roi.layers[layer_idx].get_smoothed_normals()
 
-        for pt,normal in zip(layer_centers, layer_normals):
+        for pt, normal in zip(layer_centers, layer_normals):
             for idx in range(len(intersectors)):
                 intersection_pts = vtk.vtkPoints()
                 intersected_tris = vtk.vtkIdList()
 
                 intersector = intersectors[idx]
 
-                intersector.IntersectWithLine(pt, normal_sign[idx]*100*normal+pt, intersection_pts, intersected_tris)
+                intersector.IntersectWithLine(pt, normal_sign[idx] * 100 * normal + pt, intersection_pts,
+                                              intersected_tris)
                 if intersection_pts.GetNumberOfPoints() > 0:
                     intersec_pts[idx].append(intersection_pts.GetPoint(0))
                 else:
-                    intersec_pts[idx].append([np.iinfo(np.uint16).max]*3)
+                    intersec_pts[idx].append([np.iinfo(np.uint16).max] * 3)
 
         gm_intersec_pts = np.array(intersec_pts[0])
         wm_intersec_pts = np.array(intersec_pts[1])
@@ -209,8 +227,10 @@ def precompute_geo_info_for_layer_field_interpolation(simnibs_mesh, roi):
         raycast_error_nodes_gm = np.argwhere(layer_gm_distance > 2)  # mm
         raycast_error_nodes_wm = np.argwhere(layer_wm_distance > 2)  # mm
 
-        closest_gm_nodes, _ = gray_matter_surface.nodes.find_closest_node(layer_centers[raycast_error_nodes_gm], return_index=True)
-        closest_wm_nodes, _ = white_matter_surface.nodes.find_closest_node(layer_centers[raycast_error_nodes_wm], return_index=True)
+        closest_gm_nodes, _ = gray_matter_surface.nodes.find_closest_node(layer_centers[raycast_error_nodes_gm],
+                                                                          return_index=True)
+        closest_wm_nodes, _ = white_matter_surface.nodes.find_closest_node(layer_centers[raycast_error_nodes_wm],
+                                                                           return_index=True)
 
         gm_intersec_pts[raycast_error_nodes_gm] = closest_gm_nodes
         wm_intersec_pts[raycast_error_nodes_wm] = closest_wm_nodes
@@ -222,49 +242,51 @@ def precompute_geo_info_for_layer_field_interpolation(simnibs_mesh, roi):
         center_to_wm_vecs = wm_intersec_pts - layer_centers
 
         associated_gray_matter_points = gm_intersec_pts - \
-                                        np.multiply( # multiply unit-normals by individual gm thickness, then scale to 20%
+                                        np.multiply(
+                                            # multiply unit-normals by individual gm thickness, then scale to 20%
                                             center_to_gm_vecs,
                                             layer_gm_distance[:, np.newaxis]
                                         ) * 0.1
         associated_white_matter_points = wm_intersec_pts - \
-                                         np.multiply( # multiply unit-normals by individual wm thickness, then scale to 20%
+                                         np.multiply(
+                                             # multiply unit-normals by individual wm thickness, then scale to 20%
                                              center_to_wm_vecs,
                                              layer_wm_distance[:, np.newaxis]
                                          ) * 0.1
 
         layer_gm_wm_info[layer_id] = {
-            "assoc_gm_points" : associated_gray_matter_points,
-            "assoc_wm_points" : associated_white_matter_points,
-            "layer_gm_dist" : layer_gm_distance,
-            "layer_wm_dist" : layer_wm_distance
+            "assoc_gm_points": associated_gray_matter_points,
+            "assoc_wm_points": associated_white_matter_points,
+            "layer_gm_dist": layer_gm_distance,
+            "layer_wm_dist": layer_wm_distance
         }
 
     return layer_gm_wm_info
 
 
 def calc_e_in_midlayer_roi(
-    phi_dadt,
-    roi,
-    subject,
-    phi_scaling=1.,
-    mesh_idx=0,
-    mesh=None,
-    roi_hem='lh',
-    depth=.5,
-    qoi=None,
-    layer_gm_wm_info=None
+        E,
+        roi,
+        subject,
+        phi_scaling=1.,
+        mesh_idx=0,
+        mesh=None,
+        roi_hem='lh',
+        depth=.5,
+        qoi=None,
+        layer_gm_wm_info=None
 ):
     """
     This is to be called by Simnibs as postprocessing function per FEM solve.
 
     Parameters
     ----------
-    phi_dadt : (array-like, elmdata)
-    roi : pynibs.roi.RegionOfInterestSurface()
-    subject : pynibs.subject.Subject()
+    E : (array-like, elmdata)
+    roi : pynibs.roi.RegionOfInterestSurface
+    subject : pynibs.subject.Subject
     phi_scaling : float
     mesh_idx : int
-    mesh : simnibs.msh.Mesh()
+    mesh : simnibs.msh.Mesh
     roi_hem : 'lh'
     depth : float
     qoi : list of str
@@ -272,19 +294,16 @@ def calc_e_in_midlayer_roi(
     layer_gm_wm_info : dict[str, dict[str,np.ndarray]]
         For each layer defined in the mesh (outer dict key: layer_id),
         provide pre-computed geometrical information as a dictionary (outer dict value):
-        - key: gm_nodes
-          For each layer node, the corresponding point on the GM surface.
-        - key: wm_nodes
-          For each layer node, the corresponding point on the WM surface.
-        - key: gm_center_distance
-          The distance between the layer nodes and the corresponding points on the GM.
-        - key: wm_center_distance
-          The distance between the layer nodes and the corresponding points on the WM.
+
+        * key: gm_nodes, For each layer node, the corresponding point on the GM surface.
+        * key: wm_nodes, For each layer node, the corresponding point on the WM surface.
+        * key: gm_center_distance, The distance between the layer nodes and the corresponding points on the GM.
+        * key: wm_center_distance, The distance between the layer nodes and the corresponding points on the WM.
 
     Returns
     -------
     (roi.n_tris,4) : np.vstack((e_mag, e_norm, e_tan, e_angle)).transpose() for the midlayer
-    (len(roi.layers)x(roi.layers[idx].surface.n_tris,4)) : np.vstack((e_mag, e_norm, e_tan, e_angle)).transpose()
+        (len(roi.layers)x(roi.layers[idx].surface.n_tris,4)) : np.vstack((e_mag, e_norm, e_tan, e_angle)).transpose()
     """
     if depth != .5:
         raise NotImplementedError(f"depth={depth} != 0.5 not implemented")
@@ -299,13 +318,13 @@ def calc_e_in_midlayer_roi(
     ret_arr_num_components = len(qoi)
     # special care for E
     if 'E' in qoi:
-        ret_arr_num_components += 2         # E is a 3D quantity whose components are stored individually
+        ret_arr_num_components += 2  # E is a 3D quantity whose components are stored individually
         qois_midlayer.remove('E')  # we don't need this to be computed, it already is
 
-    qois_layers = []            # the QOIs computed for the cortical layers (not midlayer)
+    qois_layers = []  # the QOIs computed for the cortical layers (not midlayer)
     max_num_elmts = roi.n_tris  # number of midlayer elements to define the shape of the output data structure
     interpolation_surfaces = {  # initialize the ROI surfaces the QOIs will be interpolated onto
-        "midlayer" : simnibs.Msh(
+        "midlayer": simnibs.Msh(
             nodes=simnibs.Nodes(node_coord=roi.node_coord_mid),
             elements=simnibs.Elements(triangles=roi.node_number_list + 1)
         )
@@ -313,7 +332,7 @@ def calc_e_in_midlayer_roi(
     # check if interpolation must be done for the cortical layers and initialize accordingly
     if len(roi.layers) > 0 and layer_gm_wm_info is not None:
         if len(layer_gm_wm_info.keys()) == len(roi.layers):
-            qois_layers = ['mag', 'theta', 'gradient'] # = parameters of the neuronal meanfield model
+            qois_layers = ['mag', 'theta', 'gradient']  # = parameters of the neuronal meanfield model
 
             for layer in roi.layers:
                 # Do the cortical layers have more elements than the midlayer?
@@ -328,64 +347,44 @@ def calc_e_in_midlayer_roi(
                   "found in the mesh. Skipping computation of neuronal model parameters...")
 
     # simnibs calls this with empty data so find out about the results dimensions
-    if isinstance(phi_dadt, tuple):
+    if isinstance(E, tuple):
         # SimNIBS 4 includes 'phi/v' last in the tuple of the to-be computed fields
-        phi = phi_dadt[-1] * phi_scaling
+        pass
         # dadt = phi_dadt[1].elm_data2node_data().value
     else:
         return np.zeros((max_num_elmts, ret_arr_num_components))
 
-    # prepare mesh_local
-    mesh_local = copy.deepcopy(mesh)
+    # calc QOIs (all calculations are performed in the element centers)
+    data = simnibs.ElementData(E[0], name='E', mesh=mesh)
 
-    # write phi and dAdt in msh
-    # dadt = mesh_io.NodeData(dadt, name='D', mesh=mesh_local)
-    try:
-        dadt = phi_dadt[0].value
-    except AttributeError:
-        dadt = phi_dadt[0]
-    dadt = simnibs.ElementData(dadt, name='D', mesh=mesh_local)
-    phi = simnibs.NodeData(phi.flatten(), name='v', mesh=mesh_local)
-
-    mesh_local = fem.calc_fields(phi, "vDEe", cond=None, dadt=dadt)
-
-    # get folder names and such
-    # m2m_folder = os.path.join(mesh_folder, "m2m_" + subject.id)
-    # m2m_folder = os.path.abspath(os.path.normpath(m2m_folder))
-
-    # if v_simnibs == 3:
-    #     names, segtype = get_surface_names_from_folder_structure(m2m_folder)
-    #     # middle_surf = {}
-    #
-    #     # get midlayer
-    #     if segtype == 'mri2mesh':
-    #         # for hemi in ['lh', 'rh']:
-    #         wm_surface = mesh_io.read_freesurfer_surface(names[roi_hem + '_wm'])
-    #         gm_surface = mesh_io.read_freesurfer_surface(names[roi_hem + '_gm'])
-    #         # middle_surf = mesh_io._middle_surface(wm_surface, gm_surface, depth)
-    #     elif segtype == 'headreco':
-    #         pass
-    #         # for hemi in ['lh', 'rh']:
-    #         # middle_surf = mesh_io.read_gifti_surface(names[roi_hem + '_midgm'])
-    #     else:
-    #         raise NotImplementedError(f"segtype {segtype} unknown.")
-
-    # calc QOIs
-    data = mesh_local.field['E']
-    data = data.as_nodedata()
     interp_res_per_surface = dict()
+    mesh.elmdata.append(simnibs.ElementData(E[0], name='E', mesh=mesh))
 
     for id, surface in interpolation_surfaces.items():
-        interpolated = data.interpolate_to_surface(surface)
-        qois_calculated = collections.OrderedDict() # maintain order of calculation with OrderedDict
+        barycenter_surface = simnibs.Msh(
+            nodes=simnibs.Nodes(node_coord=surface.elements_baricenters().value),
+            elements=None
+        )
+
+        # determine surface normals in elements
+        p1_tri = surface.nodes.node_coord[surface.elm.node_number_list[:, 0] - 1, :]
+        p2_tri = surface.nodes.node_coord[surface.elm.node_number_list[:, 1] - 1, :]
+        p3_tri = surface.nodes.node_coord[surface.elm.node_number_list[:, 2] - 1, :]
+
+        triangles_normals = np.cross(p2_tri - p1_tri, p3_tri - p1_tri).T
+        triangles_normals /= np.linalg.norm(triangles_normals, axis=0)
+        triangles_normals = triangles_normals.T
+
+        interpolated = data.interpolate_to_surface(barycenter_surface)
+        qois_calculated = collections.OrderedDict()  # maintain order of calculation with OrderedDict
 
         if id == "midlayer":
             qois_to_calc = qois_midlayer
             # E is the result of the FEM; all other quantities are secondary QOIs computed from E
             if 'E' in qoi:
                 qois_calculated['E'] = interpolated.value
-                qois_calculated['E'] = pynibs.data_nodes2elements(data=qois_calculated['E'],
-                                                                  con=(surface.elm.node_number_list - 1))
+                # qois_calculated['E'] = pynibs.data_nodes2elements(data=qois_calculated['E'],
+                #                                                   con=(surface.elm.node_number_list - 1))
         elif id != "midlayer" and len(qois_layers) > 0:
             qois_to_calc = qois_layers
         else:
@@ -395,18 +394,20 @@ def calc_e_in_midlayer_roi(
             if quant == 'mag':
                 qois_calculated[quant] = interpolated.norm().value
             elif quant == 'norm':
-                qois_calculated[quant] = interpolated.normal().value
+                qois_calculated[quant] = np.sum(qois_calculated["E"] * triangles_normals, axis=1)
                 qois_calculated[quant] *= -1
             elif quant == 'tan':
-                qois_calculated[quant] = interpolated.tangent().value
+                e_n = np.sum(qois_calculated["E"] * triangles_normals, axis=1)
+                e_mag = np.linalg.norm(qois_calculated["E"], axis=1)
+                qois_calculated[quant] = np.sqrt(e_mag ** 2 - e_n ** 2)
             elif quant == 'angle':
                 qois_calculated[quant] = interpolated.angle().value
             elif quant == 'theta':  # basically 'angle' but we want it in degrees for the meanfield model
-                qois_calculated[quant] = e_field_angle_theta(surface, mesh_local).value
+                qois_calculated[quant] = e_field_angle_theta(surface, mesh).value
             elif quant == 'gradient':
                 qois_calculated[quant] = e_field_gradient_between_wm_gm(
                     surface,
-                    mesh_local,
+                    mesh,
                     layer_gm_wm_info[id]["assoc_gm_points"],
                     layer_gm_wm_info[id]["assoc_wm_points"],
                     layer_gm_wm_info[id]["layer_gm_dist"],
@@ -415,12 +416,6 @@ def calc_e_in_midlayer_roi(
             else:
                 raise ValueError('Invalid quantity: {0}'.format(quant))
 
-            # pos-processing of the QOIs
-            # - transform point data to element data, theta and gradient already are element data.
-            if quant not in ["theta", "gradient"]:
-                qois_calculated[quant] = pynibs.data_nodes2elements(data=qois_calculated[quant],
-                                                                    con=(surface.elm.node_number_list-1))
-
             # - wrap 1D data in array
             try:
                 if qois_calculated[quant].ndim == 1:
@@ -431,15 +426,16 @@ def calc_e_in_midlayer_roi(
         # OrderedDict has maintained the order of the QOI-list.
         interp_res_per_surface[id] = [*qois_calculated.values()]
 
-    del qois_calculated, surface, interpolated, mesh_local, phi, mesh, data
+    del qois_calculated, surface, interpolated, mesh, data
     gc.collect()
 
-    # 0-pad midlayer & layer data to have the same number of columns (hstack not possible otherwise),
+    # 0-pad midlayer & layer data to have the same number of rows (hstack not possible otherwise),
     # according to the layer with the highest number of elements.
     for id in interp_res_per_surface.keys():
-        num_values_to_be_padded = max_num_elmts  - interp_res_per_surface[id][0].shape[0]
+        num_values_to_be_padded = max_num_elmts - interp_res_per_surface[id][0].shape[0]
         for q in range(len(interp_res_per_surface[id])):
-            interp_res_per_surface[id][q] = np.pad(interp_res_per_surface[id][q], ((0,num_values_to_be_padded),(0,0)))
+            interp_res_per_surface[id][q] = np.pad(interp_res_per_surface[id][q],
+                                                   ((0, num_values_to_be_padded), (0, 0)))
 
     # Stack all result fields column-wise; each column = 1-result array (or in case of E, a component of E)
     # For midlayer + 5 layers, there will be 21 columns and max_num_elements rows
@@ -459,9 +455,9 @@ def read_coil_geo(fn_coil_geo):
     Returns
     -------
     dipole_pos : np.ndarray of float
-    (n_dip, 3) Dipole positions ``(x, y, z)``.
+        (n_dip, 3) Dipole positions ``(x, y, z)``.
     dipole_mag : np.ndarray of float
-    (n_dip, 1) Dipole magnitude.
+        (n_dip, 1) Dipole magnitude.
     """
     regex = r"SP\((.*?)\)\{(.*?)\}"
     with open(fn_coil_geo, 'r') as file:
@@ -491,6 +487,7 @@ def read_coil_geo(fn_coil_geo):
 def check_mesh(mesh, verbose=False):
     """
     Check a simmibs.Mesh for degenerated elements:
+
         * zero surface triangles
         * zerso volume tetrahedra
         * negative volume tetrahedra
@@ -591,11 +588,11 @@ def smooth_mesh(mesh, output_fn, smooth=.8, approach='taubin', skin_only_output=
     mesh : str or simnibs.Mesh
     output_fn : str
     smooth : float, default: 0.8
-        Smoothing aggressiveness. [0, ..., 1]
+        Smoothing aggressiveness. ``[0, ..., 1]``.
     approach: str, default: 'taubin'
-        Which smoothing approach to use. One of ('taubin', laplacian', 'humphrey)
+        Which smoothing approach to use. One of (``'taubin'``, ``'laplacian'``, ``'humphrey'``.)
     smooth_tissue : str or list of int, default: 'skin'
-        Which tissue type to smooth. E.g. 'gm' or [2, 1002].
+        Which tissue type to smooth. E.g. ``'gm'`` or ``[2, 1002]``.
 
     Other parameters
     ----------------
@@ -606,7 +603,6 @@ def smooth_mesh(mesh, output_fn, smooth=.8, approach='taubin', skin_only_output=
     -------
     <file> : The smoothed mesh.
 
-
     .. figure:: ../../doc/images/smooth_mesh.png
        :scale: 50 %
        :alt: Original and smoothed surfaces and volumes.
