@@ -7,6 +7,9 @@ import os
 
 
 class TestSubject(unittest.TestCase):
+    """
+    Test pynibs.subject.* functions.
+    """
     subject_id = 'testsub'
     mesh_names = ['charm_refm1_sm', 'headreco', 'headreco_refined_m1']
     create_subject_fn = "create_subject_testsub.py"
@@ -27,24 +30,42 @@ class TestSubject(unittest.TestCase):
         for mesh_id in self.mesh_names:
             assert mesh_id in subject.mesh.keys(), f"{mesh_id} not found in {self.fn_subject_object}"
 
-    def test_02_save_subject(self):
+    def test_03_load_subject_h5(self):
+        no_hdf5_suffix = os.path.join(self.folder.name, 'test.h5')
+        shutil.copyfile(self.fn_subject_object, no_hdf5_suffix)
+        with self.assertRaises(NotImplementedError):
+            pynibs.load_subject(no_hdf5_suffix)
+        subject = pynibs.load_subject(no_hdf5_suffix, filetype='hdf5')
+        for mesh_id in self.mesh_names:
+            assert mesh_id in subject.mesh.keys(), f"{mesh_id} not found in {self.fn_subject_object}"
+
+    def test_04_save_subject(self):
         subject = pynibs.load_subject(self.fn_subject_object)
         fn_new = self.fn_subject_object.replace('.hdf5', '_new.hdf5')
-        pynibs.save_subject_hdf5(subject_id=self.subject_id,
-                                 subject_folder=self.folder.name,
-                                 fname=fn_new,
-                                 mri_dict=subject.mri,
-                                 mesh_dict=subject.mesh,
-                                 roi_dict=subject.roi,
-                                 exp_dict=subject.exp,
-                                 ps_dict=subject.ps,
-                                 overwrite=True,
-                                 check_file_exist=False,
-                                 verbose=False)
+        pynibs.save_subject(subject_id=self.subject_id,
+                            subject_folder=self.folder.name,
+                            fname=fn_new,
+                            mri_dict=subject.mri,
+                            mesh_dict=subject.mesh,
+                            roi_dict=subject.roi,
+                            exp_dict=subject.exp,
+                            ps_dict=subject.ps,
+                            overwrite=True,
+                            check_file_exist=False,
+                            verbose=False)
         subject = pynibs.load_subject(fn_new)
         for mesh_id in self.mesh_names:
             assert mesh_id in subject.mesh.keys(), f"{mesh_id} not found in {self.fn_subject_object}"
 
-    def test_04_print_subject(self):
+    # def test_06_add_stuff(self):
+    #     subject = pynibs.load_subject(self.fn_subject_object)
+    #     mri = subject.mri[0]
+    #     subject.add_mri_info(mri)
+    #
+    #     exp = {'TMS_localite': subject.exp['TMS_localite']}
+    #     exp['fn_coil'] = exp['TMS_localite']['fn_coil'][0][0]
+    #     subject.add_experiment_info(exp)
+
+    def test_05_print_subject(self):
         subject = pynibs.load_subject(self.fn_subject_object)
         print(subject)
